# Copyright 2002 Ben Escoto
#
# This file is part of rdiff-backup.
#
# rdiff-backup is free software; you can redistribute it and/or modify
# under the terms of the GNU General Public License as published by the
# Free Software Foundation; either version 2 of the License, or (at your
# option) any later version.
#
# rdiff-backup is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with rdiff-backup; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
# USA

"""Operations on Iterators of Read Only Remote Paths

The main structure will be an iterator that yields RORPaths.
Every RORPath has a "raw" form that makes it more amenable to
being turned into a file.  The raw form of the iterator yields
each RORPath in the form of the tuple (index, data_dictionary,
files), where files is the number of files attached (usually 1 or
0).  After that, if a file is attached, it yields that file.

"""

from __future__ import generators
import os, tempfile, UserList, types
import Globals, rpath, iterfile, log


def CollateIterators(*rorp_iters):
	"""Collate RORPath iterators by index

	So it takes two or more iterators of rorps and returns an
	iterator yielding tuples like (rorp1, rorp2) with the same
	index.  If one or the other lacks that index, it will be None

	"""
	# overflow[i] means that iter[i] has been exhausted
	# rorps[i] is None means that it is time to replenish it.
	iter_num = len(rorp_iters)
	if iter_num == 2:
		return Collate2Iters(rorp_iters[0], rorp_iters[1])
	overflow = [None] * iter_num
	rorps = overflow[:]

	def setrorps(overflow, rorps):
		"""Set the overflow and rorps list"""
		for i in range(iter_num):
			if not overflow[i] and rorps[i] is None:
				try: rorps[i] = rorp_iters[i].next()
				except StopIteration:
					overflow[i] = 1
					rorps[i] = None

	def getleastindex(rorps):
		"""Return the first index in rorps, assuming rorps isn't empty"""
		return min(map(lambda rorp: rorp.index,
					   filter(lambda x: x, rorps)))

	def yield_tuples(iter_num, overflow, rorps):
		while 1:
			setrorps(overflow, rorps)
			if not None in overflow: break

			index = getleastindex(rorps)
			yieldval = []
			for i in range(iter_num):
				if rorps[i] and rorps[i].index == index:
					yieldval.append(rorps[i])
					rorps[i] = None
				else: yieldval.append(None)
			yield IndexedTuple(index, yieldval)
	return yield_tuples(iter_num, overflow, rorps)

def Collate2Iters(riter1, riter2):
	"""Special case of CollateIterators with 2 arguments

	This does the same thing but is faster because it doesn't have
	to consider the >2 iterator case.  Profiler says speed is
	important here.

	"""
	relem1, relem2 = None, None
	while 1:
		if not relem1:
			try: relem1 = riter1.next()
			except StopIteration:
				if relem2: yield (None, relem2)
				for relem2 in riter2:
					yield (None, relem2)
				break
			index1 = relem1.index
		if not relem2:
			try: relem2 = riter2.next()
			except StopIteration:
				if relem1: yield (relem1, None)
				for relem1 in riter1:
					yield (relem1, None)
				break
			index2 = relem2.index

		if index1 < index2:
			yield (relem1, None)
			relem1 = None
		elif index1 == index2:
			yield (relem1, relem2)
			relem1, relem2 = None, None
		else: # index2 is less
			yield (None, relem2)
			relem2 = None


class IndexedTuple(UserList.UserList):
	"""Like a tuple, but has .index

	This is used by CollateIterator above, and can be passed to the
	IterTreeReducer.

	"""
	def __init__(self, index, sequence):
		self.index = index
		self.data = tuple(sequence)

	def __len__(self): return len(self.data)

	def __getitem__(self, key):
		"""This only works for numerical keys (easier this way)"""
		return self.data[key]

	def __lt__(self, other): return self.__cmp__(other) == -1
	def __le__(self, other): return self.__cmp__(other) != 1
	def __ne__(self, other): return not self.__eq__(other)
	def __gt__(self, other): return self.__cmp__(other) == 1
	def __ge__(self, other): return self.__cmp__(other) != -1
	
	def __cmp__(self, other):
		assert isinstance(other, IndexedTuple)
		if self.index < other.index: return -1
		elif self.index == other.index: return 0
		else: return 1

	def __eq__(self, other):
		if isinstance(other, IndexedTuple):
			return self.index == other.index and self.data == other.data
		elif type(other) is types.TupleType:
			return self.data == other
		else: return None

	def __str__(self):
		return  "(%s).%s" % (", ".join(map(str, self.data)), self.index)


def FillInIter(rpiter, rootrp):
	"""Given ordered rpiter and rootrp, fill in missing indicies with rpaths

	For instance, suppose rpiter contains rpaths with indicies (),
	(1,2), (2,5).  Then return iter with rpaths (), (1,), (1,2), (2,),
	(2,5).  This is used when we need to process directories before or
	after processing a file in that directory.

	"""
	# Handle first element as special case
	first_rp = rpiter.next() # StopIteration gets passed upwards
	cur_index = first_rp.index
	for i in range(len(cur_index)): yield rootrp.new_index(cur_index[:i])
	yield first_rp
	del first_rp
	old_index = cur_index

	# Now do all the other elements
	for rp in rpiter:
		cur_index = rp.index
		if not cur_index[:-1] == old_index[:-1]: # Handle special case quickly
			for i in range(1, len(cur_index)): # i==0 case already handled
				if cur_index[:i] != old_index[:i]:
					filler_rp = rootrp.new_index(cur_index[:i])
					if not filler_rp.isdir():
						log.Log("Warning: expected %s to be a directory but "
								"found %s instead.\nThis is probably caused "
								"by a bug in versions 1.0.0 and earlier." %
								(filler_rp.path, filler_rp.lstat()), 2)
						filler_rp.make_zero_dir(rootrp)
					yield filler_rp
		yield rp
		old_index = cur_index


class IterTreeReducer:
	"""Tree style reducer object for iterator

	The indicies of a RORPIter form a tree type structure.  This class
	can be used on each element of an iter in sequence and the result
	will be as if the corresponding tree was reduced.  This tries to
	bridge the gap between the tree nature of directories, and the
	iterator nature of the connection between hosts and the temporal
	order in which the files are processed.

	"""
	def __init__(self, branch_class, branch_args):
		"""ITR initializer"""
		self.branch_class = branch_class
		self.branch_args = branch_args
		self.index = None
		self.root_branch = branch_class(*branch_args)
		self.branches = [self.root_branch]
		self.root_fast_processed = None

	def finish_branches(self, index):
		"""Run Finish() on all branches index has passed

		When we pass out of a branch, delete it and process it with
		the parent.  The innermost branches will be the last in the
		list.  Return None if we are out of the entire tree, and 1
		otherwise.

		"""
		branches = self.branches
		while 1:
			to_be_finished = branches[-1]
			base_index = to_be_finished.base_index
			if base_index != index[:len(base_index)]:
				# out of the tree, finish with to_be_finished
				to_be_finished.end_process()
				del branches[-1]
				if not branches: return None
				branches[-1].branch_process(to_be_finished)
			else: return 1

	def add_branch(self, index):
		"""Return branch of type self.branch_class, add to branch list"""
		branch = self.branch_class(*self.branch_args)
		branch.base_index = index
		self.branches.append(branch)
		return branch

	def Finish(self):
		"""Call at end of sequence to tie everything up"""
		if self.index is None or self.root_fast_processed: return
		while 1:
			to_be_finished = self.branches.pop()
			to_be_finished.end_process()
			if not self.branches: break
			self.branches[-1].branch_process(to_be_finished)

	def __call__(self, *args):
		"""Process args, where args[0] is current position in iterator

		Returns true if args successfully processed, false if index is
		not in the current tree and thus the final result is
		available.

		Also note below we set self.index after doing the necessary
		start processing, in case there is a crash in the middle.

		"""
		index = args[0]
		if self.index is None:
			self.root_branch.base_index = index
			if self.root_branch.can_fast_process(*args):
				self.root_branch.fast_process(*args)
				self.root_fast_processed = 1
			else: self.root_branch.start_process(*args)
			self.index = index
			return 1
		if index == self.index:
			log.Log("Warning, repeated index %s, bad filesystem?"
					% (index,), 2)
		elif index < self.index:
			assert 0, "Bad index order: %s >= %s" % (self.index, index)
		else: # normal case
			if self.finish_branches(index) is None:
				return None # We are no longer in the main tree
			last_branch = self.branches[-1]
			if last_branch.can_fast_process(*args):
				last_branch.fast_process(*args)
			else:
				branch = self.add_branch(index)
				branch.start_process(*args)

		self.index = index
		return 1


class ITRBranch:
	"""Helper class for IterTreeReducer above

	There are five stub functions below: start_process, end_process,
	branch_process, can_fast_process, and fast_process.  A class that
	subclasses this one will probably fill in these functions to do
	more.

	"""
	base_index = index = None
	def start_process(self, *args):
		"""Do some initial processing (stub)"""
		pass

	def end_process(self):
		"""Do any final processing before leaving branch (stub)"""
		pass

	def branch_process(self, branch):
		"""Process a branch right after it is finished (stub)"""
		pass

	def can_fast_process(self, *args):
		"""True if object can be processed without new branch (stub)"""
		return None

	def fast_process(self, *args):
		"""Process args without new child branch (stub)"""
		pass


class CacheIndexable:
	"""Cache last few indexed elements in iterator

	This class should be initialized with an iterator yielding
	.index'd objects.  It looks like it is just the same iterator as
	the one that initialized it.  Luckily, it does more, caching the
	last few elements iterated, which can be retrieved using the
	.get() method.

	If the index is not in the cache, return None.

	"""
	def __init__(self, indexed_iter, cache_size = None):
		"""Make new CacheIndexable.  Cache_size is max cache length"""
		self.cache_size = cache_size
		self.iter = indexed_iter
		self.cache_dict = {}
		self.cache_indicies = []

	def next(self):
		"""Return next elem, add to cache.  StopIteration passed upwards"""
		next_elem = self.iter.next()
		next_index = next_elem.index
		self.cache_dict[next_index] = next_elem
		self.cache_indicies.append(next_index)

		if len(self.cache_indicies) > self.cache_size:
			try:
				del self.cache_dict[self.cache_indicies[0]]
			except KeyError:
				log.Log("Warning: index %s missing from iterator cache" %
					(self.cache_indicies[0],), 2)
			del self.cache_indicies[0]

		return next_elem

	def __iter__(self): return self

	def get(self, index):
		"""Return element with index index from cache"""
		try: return self.cache_dict[index]
		except KeyError:
			assert index >= self.cache_indicies[0], \
				   "Index out of order: "+repr((index, self.cache_indicies[0]))
			return None



# Copyright 2002, 2003 Ben Escoto
#
# This file is part of rdiff-backup.
#
# rdiff-backup is free software; you can redistribute it and/or modify
# under the terms of the GNU General Public License as published by the
# Free Software Foundation; either version 2 of the License, or (at your
# option) any later version.
#
# rdiff-backup is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with rdiff-backup; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
# USA

"""High level functions for mirroring and mirror+incrementing"""

from __future__ import generators
import errno
import Globals, metadata, rorpiter, TempFile, Hardlink, robust, increment, \
	   rpath, static, log, selection, Time, Rdiff, statistics, iterfile, \
	   hash, longname

def Mirror(src_rpath, dest_rpath):
	"""Turn dest_rpath into a copy of src_rpath"""
	log.Log("Starting mirror %s to %s" % (src_rpath.path, dest_rpath.path), 4)
	SourceS = src_rpath.conn.backup.SourceStruct
	DestS = dest_rpath.conn.backup.DestinationStruct

	source_rpiter = SourceS.get_source_select()
	DestS.set_rorp_cache(dest_rpath, source_rpiter, 0)
	dest_sigiter = DestS.get_sigs(dest_rpath)
	source_diffiter = SourceS.get_diffs(dest_sigiter)
	DestS.patch(dest_rpath, source_diffiter)

def Mirror_and_increment(src_rpath, dest_rpath, inc_rpath):
	"""Mirror + put increments in tree based at inc_rpath"""
	log.Log("Starting increment operation %s to %s" %
			(src_rpath.path, dest_rpath.path), 4)
	SourceS = src_rpath.conn.backup.SourceStruct
	DestS = dest_rpath.conn.backup.DestinationStruct

	source_rpiter = SourceS.get_source_select()
	DestS.set_rorp_cache(dest_rpath, source_rpiter, 1)
	dest_sigiter = DestS.get_sigs(dest_rpath)
	source_diffiter = SourceS.get_diffs(dest_sigiter)
	DestS.patch_and_increment(dest_rpath, source_diffiter, inc_rpath)


class SourceStruct:
	"""Hold info used on source side when backing up"""
	_source_select = None # will be set to source Select iterator
	def set_source_select(cls, rpath, tuplelist, *filelists):
		"""Initialize select object using tuplelist

		Note that each list in filelists must each be passed as
		separate arguments, so each is recognized as a file by the
		connection.  Otherwise we will get an error because a list
		containing files can't be pickled.

		Also, cls._source_select needs to be cached so get_diffs below
		can retrieve the necessary rps.

		"""
		sel = selection.Select(rpath)
		sel.ParseArgs(tuplelist, filelists)
		sel.set_iter()
		cache_size = Globals.pipeline_max_length * 3 # to and from+leeway
		cls._source_select = rorpiter.CacheIndexable(sel, cache_size)
		Globals.set('select_mirror', sel)

	def get_source_select(cls):
		"""Return source select iterator, set by set_source_select"""
		return cls._source_select

	def get_diffs(cls, dest_sigiter):
		"""Return diffs of any files with signature in dest_sigiter"""
		source_rps = cls._source_select
		error_handler = robust.get_error_handler("ListError")
		def attach_snapshot(diff_rorp, src_rp):
			"""Attach file of snapshot to diff_rorp, w/ error checking"""
			fileobj = robust.check_common_error(
				error_handler, rpath.RPath.open, (src_rp, "rb"))
			if fileobj: diff_rorp.setfile(hash.FileWrapper(fileobj))
			else: diff_rorp.zero()
			diff_rorp.set_attached_filetype('snapshot')

		def attach_diff(diff_rorp, src_rp, dest_sig):
			"""Attach file of diff to diff_rorp, w/ error checking"""
			fileobj = robust.check_common_error(
				error_handler, Rdiff.get_delta_sigrp_hash, (dest_sig, src_rp))
			if fileobj:
				diff_rorp.setfile(fileobj)
				diff_rorp.set_attached_filetype('diff')
			else:
				diff_rorp.zero()
				diff_rorp.set_attached_filetype('snapshot')
				
		for dest_sig in dest_sigiter:
			if dest_sig is iterfile.MiscIterFlushRepeat:
				yield iterfile.MiscIterFlush # Flush buffer when get_sigs does
				continue
			src_rp = (source_rps.get(dest_sig.index) or
					  rpath.RORPath(dest_sig.index))
			diff_rorp = src_rp.getRORPath()
			if dest_sig.isflaglinked():
				diff_rorp.flaglinked(dest_sig.get_link_flag())
			elif src_rp.isreg():
				if dest_sig.isreg(): attach_diff(diff_rorp, src_rp, dest_sig)
				else: attach_snapshot(diff_rorp, src_rp)
			else:
				dest_sig.close_if_necessary()
				diff_rorp.set_attached_filetype('snapshot')
			yield diff_rorp

static.MakeClass(SourceStruct)


class DestinationStruct:
	"""Hold info used by destination side when backing up"""
	def get_dest_select(cls, rpath, use_metadata = 1):
		"""Return destination select rorpath iterator

		If metadata file doesn't exist, select all files on
		destination except rdiff-backup-data directory.

		"""
		def get_iter_from_fs():
			"""Get the combined iterator from the filesystem"""
			sel = selection.Select(rpath)
			sel.parse_rbdir_exclude()
			return sel.set_iter()

		metadata.SetManager()
		if use_metadata:
			rorp_iter = metadata.ManagerObj.GetAtTime(Time.prevtime)
			if rorp_iter: return rorp_iter
		return get_iter_from_fs()

	def set_rorp_cache(cls, baserp, source_iter, for_increment):
		"""Initialize cls.CCPP, the destination rorp cache

		for_increment should be true if we are mirror+incrementing,
		false if we are just mirroring.

		"""
		dest_iter = cls.get_dest_select(baserp, for_increment)
		collated = rorpiter.Collate2Iters(source_iter, dest_iter)
		cls.CCPP = CacheCollatedPostProcess(
			collated, Globals.pipeline_max_length*4, baserp)
		# pipeline len adds some leeway over just*3 (to and from and back)
		
	def get_sigs(cls, dest_base_rpath):
		"""Yield signatures of any changed destination files

		If we are backing up across a pipe, we must flush the pipeline
		every so often so it doesn't get congested on destination end.

		"""
		flush_threshold = Globals.pipeline_max_length - 2
		num_rorps_seen = 0
		for src_rorp, dest_rorp in cls.CCPP:
			if (Globals.backup_reader is not Globals.backup_writer):
				num_rorps_seen += 1
				if (num_rorps_seen > flush_threshold):
					num_rorps_seen = 0
					yield iterfile.MiscIterFlushRepeat
			if not (src_rorp and dest_rorp and src_rorp == dest_rorp and
				(not Globals.preserve_hardlinks or
				 Hardlink.rorp_eq(src_rorp, dest_rorp))):

				index = src_rorp and src_rorp.index or dest_rorp.index
				sig = cls.get_one_sig(dest_base_rpath, index,
									  src_rorp, dest_rorp)
				if sig:
					cls.CCPP.flag_changed(index)
					yield sig

	def get_one_sig(cls, dest_base_rpath, index, src_rorp, dest_rorp):
		"""Return a signature given source and destination rorps"""
		if (Globals.preserve_hardlinks and src_rorp and
			Hardlink.islinked(src_rorp)):
			dest_sig = rpath.RORPath(index)
			dest_sig.flaglinked(Hardlink.get_link_index(src_rorp))
		elif dest_rorp:
			dest_sig = dest_rorp.getRORPath()
			if dest_rorp.isreg():
				dest_rp = longname.get_mirror_rp(dest_base_rpath, dest_rorp)
				sig_fp = cls.get_one_sig_fp(dest_rp)
				if sig_fp is None: return None
				dest_sig.setfile(sig_fp)
		else: dest_sig = rpath.RORPath(index)
		return dest_sig

	def get_one_sig_fp(cls, dest_rp):
		"""Return a signature fp of given index, corresponding to reg file"""
		if not dest_rp.isreg():
			log.ErrorLog.write_if_open("UpdateError", dest_rp,
				"File changed from regular file before signature")
			return None
		if (Globals.process_uid != 0 and not dest_rp.readable() and
				dest_rp.isowner()):
			# This branch can happen with root source and non-root
			# destination.  Permissions are changed permanently, which
			# should propogate to the diffs
			dest_rp.chmod(0400 | dest_rp.getperms())
		try:
			return Rdiff.get_signature(dest_rp)
		except IOError, e:
			if (e.errno == errno.EPERM):
				try:
					# Try chmod'ing anyway -- This can work on NFS and AFS
					# depending on the setup. We keep the if() statement
					# above for performance reasons.
					dest_rp.chmod(0400 | dest_rp.getperms())
					return Rdiff.get_signature(dest_rp)
				except (IOError, OSError):
					log.Log.FatalError("Could not open %s for reading. Check "
						"permissions on file." % (dest_rp.path,))
			else:
				raise

	def patch(cls, dest_rpath, source_diffiter, start_index = ()):
		"""Patch dest_rpath with an rorpiter of diffs"""
		ITR = rorpiter.IterTreeReducer(PatchITRB, [dest_rpath, cls.CCPP])
		for diff in rorpiter.FillInIter(source_diffiter, dest_rpath):
			log.Log("Processing changed file " + diff.get_indexpath(), 5)
			ITR(diff.index, diff)
		ITR.Finish()
		cls.CCPP.close()
		dest_rpath.setdata()

	def patch_and_increment(cls, dest_rpath, source_diffiter, inc_rpath):
		"""Patch dest_rpath with rorpiter of diffs and write increments"""
		ITR = rorpiter.IterTreeReducer(IncrementITRB,
									   [dest_rpath, inc_rpath, cls.CCPP])
		for diff in rorpiter.FillInIter(source_diffiter, dest_rpath):
			log.Log("Processing changed file " + diff.get_indexpath(), 5)
			ITR(diff.index, diff)
		ITR.Finish()
		cls.CCPP.close()
		dest_rpath.setdata()

static.MakeClass(DestinationStruct)


class CacheCollatedPostProcess:
	"""Cache a collated iter of (source_rorp, dest_rorp) pairs

	This is necessary for three reasons:

	1.  The patch function may need the original source_rorp or
	    dest_rp information, which is not present in the diff it
	    receives.

	2.  The metadata must match what is stored in the destination
	    directory.  If there is an error, either we do not update the
	    dest directory for that file and the old metadata is used, or
	    the file is deleted on the other end..  Thus we cannot write
	    any metadata until we know the file has been procesed
	    correctly.

	3.  We may lack permissions on certain destination directories.
	    The permissions of these directories need to be relaxed before
	    we enter them to computer signatures, and then reset after we
	    are done patching everything inside them.

	4.  We need some place to put hashes (like SHA1) after computing
	    them and before writing them to the metadata.

	The class caches older source_rorps and dest_rps so the patch
	function can retrieve them if necessary.  The patch function can
	also update the processed correctly flag.  When an item falls out
	of the cache, we assume it has been processed, and write the
	metadata for it.

	"""
	def __init__(self, collated_iter, cache_size, dest_root_rp):
		"""Initialize new CCWP."""
		self.iter = collated_iter # generates (source_rorp, dest_rorp) pairs
		self.cache_size = cache_size
		self.dest_root_rp = dest_root_rp

		self.statfileobj = statistics.init_statfileobj()
		if Globals.file_statistics: statistics.FileStats.init()
		self.metawriter = metadata.ManagerObj.GetWriter()

		# the following should map indicies to lists
		# [source_rorp, dest_rorp, changed_flag, success_flag, increment]

		# changed_flag should be true if the rorps are different, and

		# success_flag should be 1 if dest_rorp has been successfully
		# updated to source_rorp, and 2 if the destination file is
		# deleted entirely.  They both default to false (0).
		
		# increment holds the RPath of the increment file if one
		# exists.  It is used to record file statistics.
		
		self.cache_dict = {}
		self.cache_indicies = []

		# Contains a list of pairs (destination_rps, permissions) to
		# be used to reset the permissions of certain directories
		# after we're finished with them
		self.dir_perms_list = []

		# Contains list of (index, (source_rorp, diff_rorp)) pairs for
		# the parent directories of the last item in the cache.
		self.parent_list = []

	def __iter__(self): return self

	def next(self):
		"""Return next (source_rorp, dest_rorp) pair.  StopIteration passed"""
		source_rorp, dest_rorp = self.iter.next()
		self.pre_process(source_rorp, dest_rorp)
		index = source_rorp and source_rorp.index or dest_rorp.index
		self.cache_dict[index] = [source_rorp, dest_rorp, 0, 0, None]
		self.cache_indicies.append(index)

		if len(self.cache_indicies) > self.cache_size: self.shorten_cache()
		return source_rorp, dest_rorp

	def pre_process(self, source_rorp, dest_rorp):
		"""Do initial processing on source_rorp and dest_rorp

		It will not be clear whether source_rorp and dest_rorp have
		errors at this point, so don't do anything which assumes they
		will be backed up correctly.

		"""
		if Globals.preserve_hardlinks and source_rorp:
			Hardlink.add_rorp(source_rorp, dest_rorp)
		if (dest_rorp and dest_rorp.isdir() and Globals.process_uid != 0
			and dest_rorp.getperms() % 01000 < 0700):
			self.unreadable_dir_init(source_rorp, dest_rorp)

	def unreadable_dir_init(self, source_rorp, dest_rorp):
		"""Initialize an unreadable dir.

		Make it readable, and if necessary, store the old permissions
		in self.dir_perms_list so the old perms can be restored.

		"""
		dest_rp = self.dest_root_rp.new_index(dest_rorp.index)
		dest_rp.chmod(0700 | dest_rorp.getperms())
		if source_rorp and source_rorp.isdir():
			self.dir_perms_list.append((dest_rp, source_rorp.getperms()))

	def shorten_cache(self):
		"""Remove one element from cache, possibly adding it to metadata"""
		first_index = self.cache_indicies[0]
		del self.cache_indicies[0]
		try: (old_source_rorp, old_dest_rorp, changed_flag,
			  success_flag, inc) = self.cache_dict[first_index]
		except KeyError: # probably caused by error in file system (dup)
			log.Log("Warning index %s missing from CCPP cache" %
					(first_index,),2)
			return
		del self.cache_dict[first_index]
		self.post_process(old_source_rorp, old_dest_rorp,
						  changed_flag, success_flag, inc)
		if self.dir_perms_list: self.reset_dir_perms(first_index)
		self.update_parent_list(first_index, old_source_rorp, old_dest_rorp)

	def update_parent_list(self, index, src_rorp, dest_rorp):
		"""Update the parent cache with the recently expired main cache entry

		This method keeps parent directories in the secondary parent
		cache until all their children have expired from the main
		cache.  This is necessary because we may realize we need a
		parent directory's information after we have processed many
		subfiles.

		"""
		if not (src_rorp and src_rorp.isdir() or
				dest_rorp and dest_rorp.isdir()): return # neither is directory
		if not self.parent_list: assert index == (), index
		else:
			last_parent_index = self.parent_list[-1][0]
			lp_index, li = len(last_parent_index), len(index)
			if li > lp_index: # descended into previous parent
				assert li == lp_index + 1, (index, last_parent_index)
			else: # In new directory
				assert last_parent_index[:li-1] == index[:-1], index
				self.parent_list = self.parent_list[:li]
		self.parent_list.append((index, (src_rorp, dest_rorp)))

	def post_process(self, source_rorp, dest_rorp, changed, success, inc):
		"""Post process source_rorp and dest_rorp.

		The point of this is to write statistics and metadata.

		changed will be true if the files have changed.  success will
		be true if the files have been successfully updated (this is
		always false for un-changed files).

		"""
		if Globals.preserve_hardlinks and source_rorp:
			Hardlink.del_rorp(source_rorp)

		if not changed or success:
			if source_rorp: self.statfileobj.add_source_file(source_rorp)
			if dest_rorp: self.statfileobj.add_dest_file(dest_rorp)
		if success == 0: metadata_rorp = dest_rorp
		elif success == 1: metadata_rorp = source_rorp
		else: metadata_rorp = None # in case deleted because of ListError
		if success == 1 or success == 2: 
			self.statfileobj.add_changed(source_rorp, dest_rorp)

		if metadata_rorp and metadata_rorp.lstat():
			self.metawriter.write_object(metadata_rorp)
		if Globals.file_statistics:
			statistics.FileStats.update(source_rorp, dest_rorp, changed, inc)

	def reset_dir_perms(self, current_index):
		"""Reset the permissions of directories when we have left them"""
		dir_rp, perms = self.dir_perms_list[-1]
		dir_index = dir_rp.index
		if (current_index > dir_index and
			current_index[:len(dir_index)] != dir_index):
			dir_rp.chmod(perms) # out of directory, reset perms now

	def in_cache(self, index):
		"""Return true if given index is cached"""
		return self.cache_dict.has_key(index)

	def flag_success(self, index):
		"""Signal that the file with given index was updated successfully"""
		self.cache_dict[index][3] = 1

	def flag_deleted(self, index):
		"""Signal that the destination file was deleted"""
		self.cache_dict[index][3] = 2

	def flag_changed(self, index):
		"""Signal that the file with given index has changed"""
		self.cache_dict[index][2] = 1

	def set_inc(self, index, inc):
		"""Set the increment of the current file"""
		self.cache_dict[index][4] = inc

	def get_parent_rorps(self, index):
		"""Retrieve (src_rorp, dest_rorp) pair from parent cache"""
		for parent_index, pair in self.parent_list:
			if parent_index == index: return pair
		raise KeyError(index)

	def get_rorps(self, index):
		"""Retrieve (source_rorp, dest_rorp) from cache"""
		try: return self.cache_dict[index][:2]
		except KeyError: return self.get_parent_rorps(index)

	def get_source_rorp(self, index):
		"""Retrieve source_rorp with given index from cache"""
		assert index >= self.cache_indicies[0], \
			   ("CCPP index out of order: %s %s" %
				(repr(index), repr(self.cache_indicies[0])))
		try: return self.cache_dict[index][0]
		except KeyError: return self.get_parent_rorps(index)[0]

	def get_mirror_rorp(self, index):
		"""Retrieve mirror_rorp with given index from cache"""
		try: return self.cache_dict[index][1]
		except KeyError: return self.get_parent_rorps(index)[1]

	def update_hash(self, index, sha1sum):
		"""Update the source rorp's SHA1 hash"""
		self.get_source_rorp(index).set_sha1(sha1sum)

	def update_hardlink_hash(self, diff_rorp):
		"""Tag associated source_rorp with same hash diff_rorp points to"""
		sha1sum = Hardlink.get_sha1(diff_rorp)
		if not sha1sum: return
		source_rorp = self.get_source_rorp(diff_rorp.index)
		source_rorp.set_sha1(sha1sum)

	def close(self):
		"""Process the remaining elements in the cache"""
		while self.cache_indicies: self.shorten_cache()
		while self.dir_perms_list:
			dir_rp, perms = self.dir_perms_list.pop()
			dir_rp.chmod(perms)
		self.metawriter.close()
		metadata.ManagerObj.ConvertMetaToDiff()

		if Globals.print_statistics: statistics.print_active_stats()
		if Globals.file_statistics: statistics.FileStats.close()
		statistics.write_active_statfileobj()


class PatchITRB(rorpiter.ITRBranch):
	"""Patch an rpath with the given diff iters (use with IterTreeReducer)

	The main complication here involves directories.  We have to
	finish processing the directory after what's in the directory, as
	the directory may have inappropriate permissions to alter the
	contents or the dir's mtime could change as we change the
	contents.

	"""
	def __init__(self, basis_root_rp, CCPP):
		"""Set basis_root_rp, the base of the tree to be incremented"""
		self.basis_root_rp = basis_root_rp
		assert basis_root_rp.conn is Globals.local_connection
		self.statfileobj = (statistics.get_active_statfileobj() or
							statistics.StatFileObj())
		self.dir_replacement, self.dir_update = None, None
		self.CCPP = CCPP
		self.error_handler = robust.get_error_handler("UpdateError")

	def can_fast_process(self, index, diff_rorp):
		"""True if diff_rorp and mirror are not directories"""
		mirror_rorp = self.CCPP.get_mirror_rorp(index)
		return not (diff_rorp.isdir() or (mirror_rorp and mirror_rorp.isdir()))

	def fast_process(self, index, diff_rorp):
		"""Patch base_rp with diff_rorp (case where neither is directory)"""
		mirror_rp, discard = longname.get_mirror_inc_rps(
			self.CCPP.get_rorps(index), self.basis_root_rp)
		assert not mirror_rp.isdir(), mirror_rp
		tf = TempFile.new(mirror_rp)
		if self.patch_to_temp(mirror_rp, diff_rorp, tf):
			if tf.lstat():
				if robust.check_common_error(self.error_handler, rpath.rename,
						(tf, mirror_rp)) is None:
					self.CCPP.flag_success(index)
				else:
					tf.delete()
			elif mirror_rp and mirror_rp.lstat():
				mirror_rp.delete()
				self.CCPP.flag_deleted(index)
		else: 
			tf.setdata()
			if tf.lstat(): tf.delete()

	def patch_to_temp(self, basis_rp, diff_rorp, new):
		"""Patch basis_rp, writing output in new, which doesn't exist yet

		Returns true if able to write new as desired, false if
		UpdateError or similar gets in the way.

		"""
		if diff_rorp.isflaglinked():
			self.patch_hardlink_to_temp(diff_rorp, new)
		elif diff_rorp.get_attached_filetype() == 'snapshot':
			result = self.patch_snapshot_to_temp(diff_rorp, new)
			if not result: return 0
			elif result == 2: return 1 # SpecialFile
		elif not self.patch_diff_to_temp(basis_rp, diff_rorp, new):
			return 0
		if new.lstat() and not diff_rorp.isflaglinked():
			rpath.copy_attribs(diff_rorp, new)
		return self.matches_cached_rorp(diff_rorp, new)

	def patch_hardlink_to_temp(self, diff_rorp, new):
		"""Hardlink diff_rorp to temp, update hash if necessary"""
		Hardlink.link_rp(diff_rorp, new, self.basis_root_rp)
		self.CCPP.update_hardlink_hash(diff_rorp)

	def patch_snapshot_to_temp(self, diff_rorp, new):
		"""Write diff_rorp to new, return true if successful

		Returns 1 if normal success, 2 if special file is written,
		whether or not it is successful.  This is because special
		files either fail with a SpecialFileError, or don't need to be
		compared.

		"""
		if diff_rorp.isspecial():
			self.write_special(diff_rorp, new)
			rpath.copy_attribs(diff_rorp, new)
			return 2
		
		report = robust.check_common_error(self.error_handler, rpath.copy,
										   (diff_rorp, new))
		if isinstance(report, hash.Report):
			self.CCPP.update_hash(diff_rorp.index, report.sha1_digest)
			return 1
		return report != 0 # if == 0, error_handler caught something

	def patch_diff_to_temp(self, basis_rp, diff_rorp, new):
		"""Apply diff_rorp to basis_rp, write output in new"""
		assert diff_rorp.get_attached_filetype() == 'diff'
		report = robust.check_common_error(self.error_handler,
			      Rdiff.patch_local, (basis_rp, diff_rorp, new))
		if isinstance(report, hash.Report):
			self.CCPP.update_hash(diff_rorp.index, report.sha1_digest)
			return 1
		return report != 0 # if report == 0, error

	def matches_cached_rorp(self, diff_rorp, new_rp):
		"""Return true if new_rp matches cached src rorp

		This is a final check to make sure the temp file just written
		matches the stats which we got earlier.  If it doesn't it
		could confuse the regress operation.  This is only necessary
		for regular files.

		"""
		if not new_rp.isreg(): return 1
		cached_rorp = self.CCPP.get_source_rorp(diff_rorp.index)
		if cached_rorp and cached_rorp.equal_loose(new_rp): return 1
		log.ErrorLog.write_if_open("UpdateError", diff_rorp, "Updated mirror "
					  "temp file %s does not match source" % (new_rp.path,))
		return 0

	def write_special(self, diff_rorp, new):
		"""Write diff_rorp (which holds special file) to new"""
		eh = robust.get_error_handler("SpecialFileError")
		if robust.check_common_error(eh, rpath.copy, (diff_rorp, new)) == 0:
			new.setdata()
			if new.lstat(): new.delete()
			new.touch()

	def start_process(self, index, diff_rorp):
		"""Start processing directory - record information for later"""
		self.base_rp, discard = longname.get_mirror_inc_rps(
			self.CCPP.get_rorps(index), self.basis_root_rp)
		if diff_rorp.isdir(): self.prepare_dir(diff_rorp, self.base_rp)
		elif self.set_dir_replacement(diff_rorp, self.base_rp):
			if diff_rorp.lstat(): self.CCPP.flag_success(index)
			else: self.CCPP.flag_deleted(index)

	def set_dir_replacement(self, diff_rorp, base_rp):
		"""Set self.dir_replacement, which holds data until done with dir

		This is used when base_rp is a dir, and diff_rorp is not.
		Returns 1 for success or 0 for failure

		"""
		assert diff_rorp.get_attached_filetype() == 'snapshot'
		self.dir_replacement = TempFile.new(base_rp)
		if not self.patch_to_temp(None, diff_rorp, self.dir_replacement):
			if self.dir_replacement.lstat(): self.dir_replacement.delete()
			# Was an error, so now restore original directory
			rpath.copy_with_attribs(self.CCPP.get_mirror_rorp(diff_rorp.index),
									self.dir_replacement)
			return 0
		else: return 1

	def prepare_dir(self, diff_rorp, base_rp):
		"""Prepare base_rp to be a directory"""
		self.dir_update = diff_rorp.getRORPath() # make copy in case changes
		if not base_rp.isdir():
			if base_rp.lstat(): self.base_rp.delete()
			base_rp.setdata()
			base_rp.mkdir()
			self.CCPP.flag_success(diff_rorp.index)
		else: # maybe no change, so query CCPP before tagging success
			if self.CCPP.in_cache(diff_rorp.index):
				self.CCPP.flag_success(diff_rorp.index)

	def end_process(self):
		"""Finish processing directory"""
		if self.dir_update:
			assert self.base_rp.isdir()
			rpath.copy_attribs(self.dir_update, self.base_rp)

			if (Globals.process_uid != 0 and
					self.dir_update.getperms() % 01000 < 0700):
				# Directory was unreadable at start -- keep it readable
				# until the end of the backup process.
				self.base_rp.chmod(0700 | self.dir_update.getperms())
		elif self.dir_replacement:
			self.base_rp.rmdir()
			if self.dir_replacement.lstat():
				rpath.rename(self.dir_replacement, self.base_rp)


class IncrementITRB(PatchITRB):
	"""Patch an rpath with the given diff iters and write increments

	Like PatchITRB, but this time also write increments.

	"""
	def __init__(self, basis_root_rp, inc_root_rp, rorp_cache):
		self.inc_root_rp = inc_root_rp
		PatchITRB.__init__(self, basis_root_rp, rorp_cache)

	def fast_process(self, index, diff_rorp):
		"""Patch base_rp with diff_rorp and write increment (neither is dir)"""
		mirror_rp, inc_prefix = longname.get_mirror_inc_rps(
			self.CCPP.get_rorps(index), self.basis_root_rp, self.inc_root_rp)
		tf = TempFile.new(mirror_rp)
		if self.patch_to_temp(mirror_rp, diff_rorp, tf):
			inc = robust.check_common_error(self.error_handler,
					increment.Increment, (tf, mirror_rp, inc_prefix))
			if inc is not None and not isinstance(inc, int):
				self.CCPP.set_inc(index, inc)
				if inc.isreg():
					inc.fsync_with_dir() # Write inc before rp changed
				if tf.lstat():
					if robust.check_common_error(self.error_handler,
							rpath.rename, (tf, mirror_rp)) is None:
						self.CCPP.flag_success(index)
					else:
						tf.delete()
				elif mirror_rp.lstat():
					mirror_rp.delete()
					self.CCPP.flag_deleted(index)
				return # normal return, otherwise error occurred
		tf.setdata()
		if tf.lstat(): tf.delete()

	def start_process(self, index, diff_rorp):
		"""Start processing directory"""
		self.base_rp, inc_prefix = longname.get_mirror_inc_rps(
			self.CCPP.get_rorps(index), self.basis_root_rp, self.inc_root_rp)
		self.base_rp.setdata()
		assert diff_rorp.isdir() or self.base_rp.isdir(), \
				("Either %s or %s must be a directory" % (repr(diff_rorp.path),
				 repr(self.base_rp.path)))
		if diff_rorp.isdir():
			inc = increment.Increment(diff_rorp, self.base_rp, inc_prefix)
			if inc and inc.isreg():
				inc.fsync_with_dir() # must write inc before rp changed
			self.base_rp.setdata() # in case written by increment above
			self.prepare_dir(diff_rorp, self.base_rp)
		elif self.set_dir_replacement(diff_rorp, self.base_rp):
			inc = increment.Increment(self.dir_replacement, self.base_rp,
									  inc_prefix)
			if inc:
				self.CCPP.set_inc(index, inc)
				self.CCPP.flag_success(index)





# Copyright 2002 Ben Escoto
#
# This file is part of rdiff-backup.
#
# rdiff-backup is free software; you can redistribute it and/or modify
# under the terms of the GNU General Public License as published by the
# Free Software Foundation; either version 2 of the License, or (at your
# option) any later version.
#
# rdiff-backup is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with rdiff-backup; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
# USA

"""Support code for remote execution and data transfer"""

from __future__ import generators
import types, os, tempfile, cPickle, shutil, traceback, \
	   socket, sys, gzip
# The following EA and ACL modules may be used if available
try: import xattr
except ImportError: pass
try: import posix1e
except ImportError: pass
try: import win32security
except ImportError: pass

class ConnectionError(Exception): pass
class ConnectionReadError(ConnectionError): pass
class ConnectionWriteError(ConnectionError): pass
class ConnectionQuit(Exception): pass


class Connection:
	"""Connection class - represent remote execution

	The idea is that, if c is an instance of this class, c.foo will
	return the object on the remote side.  For functions, c.foo will
	return a function that, when called, executes foo on the remote
	side, sending over the arguments and sending back the result.

	"""
	def __repr__(self): return self.__str__()
	def __str__(self): return "Simple Connection" # override later
	def __nonzero__(self): return 1

class LocalConnection(Connection):
	"""Local connection

	This is a dummy connection class, so that LC.foo just evaluates to
	foo using global scope.

	"""
	def __init__(self):
		"""This prevents two instances of LocalConnection"""
		assert not Globals.local_connection
		self.conn_number = 0 # changed by SetConnections for server

	def __getattr__(self, name):
		if name in globals(): return globals()[name]
		elif isinstance(__builtins__, dict): return __builtins__[name]
		else: return __builtins__.__dict__[name]

	def __setattr__(self, name, value): globals()[name] = value

	def __delattr__(self, name): del globals()[name]

	def __str__(self): return "LocalConnection"

	def reval(self, function_string, *args):
		return apply(eval(function_string), args)

	def quit(self): pass


class ConnectionRequest:
	"""Simple wrapper around a PipeConnection request"""
	def __init__(self, function_string, num_args):
		self.function_string = function_string
		self.num_args = num_args

	def __str__(self):
		return "ConnectionRequest: %s with %d arguments" % \
			   (self.function_string, self.num_args)


class LowLevelPipeConnection(Connection):
	"""Routines for just sending objects from one side of pipe to another

	Each thing sent down the pipe is paired with a request number,
	currently limited to be between 0 and 255.  The size of each thing
	should be less than 2^56.

	Each thing also has a type, indicated by one of the following
	characters:

	o - generic object
	i - iterator/generator of RORPs
	f - file object
	b - string
	q - quit signal
	R - RPath
	Q - QuotedRPath
	r - RORPath only
	c - PipeConnection object

	"""
	def __init__(self, inpipe, outpipe):
		"""inpipe is a file-type open for reading, outpipe for writing"""
		self.inpipe = inpipe
		self.outpipe = outpipe

	def __str__(self):
		"""Return string version

		This is actually an important function, because otherwise
		requests to represent this object would result in "__str__"
		being executed on the other side of the connection.

		"""
		return "LowLevelPipeConnection"

	def _put(self, obj, req_num):
		"""Put an object into the pipe (will send raw if string)"""
		log.Log.conn("sending", obj, req_num)
		if type(obj) is types.StringType: self._putbuf(obj, req_num)
		elif isinstance(obj, connection.Connection):self._putconn(obj, req_num)
		elif isinstance(obj, FilenameMapping.QuotedRPath):
			self._putqrpath(obj, req_num)
		elif isinstance(obj, rpath.RPath): self._putrpath(obj, req_num)
		elif isinstance(obj, rpath.RORPath): self._putrorpath(obj, req_num)
		elif ((hasattr(obj, "read") or hasattr(obj, "write"))
			  and hasattr(obj, "close")): self._putfile(obj, req_num)
		elif hasattr(obj, "next"): self._putiter(obj, req_num)
		else: self._putobj(obj, req_num)

	def _putobj(self, obj, req_num):
		"""Send a generic python obj down the outpipe"""
		self._write("o", cPickle.dumps(obj, 1), req_num)

	def _putbuf(self, buf, req_num):
		"""Send buffer buf down the outpipe"""
		self._write("b", buf, req_num)

	def _putfile(self, fp, req_num):
		"""Send a file to the client using virtual files"""
		self._write("f", str(VirtualFile.new(fp)), req_num)

	def _putiter(self, iterator, req_num):
		"""Put an iterator through the pipe"""
		self._write("i",
			str(VirtualFile.new(iterfile.MiscIterToFile(iterator))), req_num)

	def _putrpath(self, rpath, req_num):
		"""Put an rpath into the pipe

		The rpath's connection will be encoded as its conn_number.  It
		and the other information is put in a tuple.

		"""
		rpath_repr = (rpath.conn.conn_number, rpath.base,
					  rpath.index, rpath.data)
		self._write("R", cPickle.dumps(rpath_repr, 1), req_num)

	def _putqrpath(self, qrpath, req_num):
		"""Put a quoted rpath into the pipe (similar to _putrpath above)"""
		qrpath_repr = (qrpath.conn.conn_number, qrpath.base,
					   qrpath.index, qrpath.data)
		self._write("Q", cPickle.dumps(qrpath_repr, 1), req_num)

	def _putrorpath(self, rorpath, req_num):
		"""Put an rorpath into the pipe

		This is only necessary because if there is a .file attached,
		it must be excluded from the pickling

		"""
		rorpath_repr = (rorpath.index, rorpath.data)
		self._write("r", cPickle.dumps(rorpath_repr, 1), req_num)

	def _putconn(self, pipeconn, req_num):
		"""Put a connection into the pipe

		A pipe connection is represented just as the integer (in
		string form) of its connection number it is *connected to*.

		"""
		self._write("c", str(pipeconn.conn_number), req_num)

	def _putquit(self):
		"""Send a string that takes down server"""
		self._write("q", "", 255)

	def _write(self, headerchar, data, req_num):
		"""Write header and then data to the pipe"""
		try:
			self.outpipe.write(headerchar + chr(req_num) +
							   C.long2str(long(len(data))))
			self.outpipe.write(data)
			self.outpipe.flush()
		except (IOError, AttributeError): raise ConnectionWriteError()

	def _read(self, length):
		"""Read length bytes from inpipe, returning result"""
		try: return self.inpipe.read(length)
		except IOError: raise ConnectionReadError()

	def _s2l_old(self, s):
		"""Convert string to long int"""
		assert len(s) == 7
		l = 0L
		for i in range(7): l = l*256 + ord(s[i])
		return l

	def _l2s_old(self, l):
		"""Convert long int to string"""
		s = ""
		for i in range(7):
			l, remainder = divmod(l, 256)
			s = chr(remainder) + s
		assert remainder == 0
		return s

	def _get(self):
		"""Read an object from the pipe and return (req_num, value)"""
		header_string = self.inpipe.read(9)
		if not len(header_string) == 9:
			raise ConnectionReadError("Truncated header string (problem "
									  "probably originated remotely)")
		format_string, req_num, length = (header_string[0],
										  ord(header_string[1]),
										  C.str2long(header_string[2:]))
		if format_string == "q": raise ConnectionQuit("Received quit signal")

		data = self._read(length)
		if format_string == "o": result = cPickle.loads(data)
		elif format_string == "b": result = data
		elif format_string == "f": result = VirtualFile(self, int(data))
		elif format_string == "i":
			result = iterfile.FileToMiscIter(VirtualFile(self, int(data)))
		elif format_string == "r": result = self._getrorpath(data)
		elif format_string == "R": result = self._getrpath(data)
		elif format_string == "Q": result = self._getqrpath(data)
		else:
			assert format_string == "c", header_string
			result = Globals.connection_dict[int(data)]
		log.Log.conn("received", result, req_num)
		return (req_num, result)

	def _getrorpath(self, raw_rorpath_buf):
		"""Reconstruct RORPath object from raw data"""
		index, data = cPickle.loads(raw_rorpath_buf)
		return rpath.RORPath(index, data)

	def _getrpath(self, raw_rpath_buf):
		"""Return RPath object indicated by raw_rpath_buf"""
		conn_number, base, index, data = cPickle.loads(raw_rpath_buf)
		return rpath.RPath(Globals.connection_dict[conn_number],
						   base, index, data)

	def _getqrpath(self, raw_qrpath_buf):
		"""Return QuotedRPath object from raw buffer"""
		conn_number, base, index, data = cPickle.loads(raw_qrpath_buf)
		return FilenameMapping.QuotedRPath(
			Globals.connection_dict[conn_number], base, index, data)

	def _close(self):
		"""Close the pipes associated with the connection"""
		self.outpipe.close()
		self.inpipe.close()


class PipeConnection(LowLevelPipeConnection):
	"""Provide server and client functions for a Pipe Connection

	Both sides act as modules that allows for remote execution.  For
	instance, self.conn.pow(2,8) will execute the operation on the
	server side.

	The only difference between the client and server is that the
	client makes the first request, and the server listens first.

	"""
	def __init__(self, inpipe, outpipe, conn_number = 0):
		"""Init PipeConnection

		conn_number should be a unique (to the session) integer to
		identify the connection.  For instance, all connections to the
		client have conn_number 0.  Other connections can use this
		number to route commands to the correct process.

		"""
		LowLevelPipeConnection.__init__(self, inpipe, outpipe)
		self.conn_number = conn_number
		self.unused_request_numbers = {}
		for i in range(256): self.unused_request_numbers[i] = None

	def __str__(self): return "PipeConnection %d" % self.conn_number

	def get_response(self, desired_req_num):
		"""Read from pipe, responding to requests until req_num.

		Sometimes after a request is sent, the other side will make
		another request before responding to the original one.  In
		that case, respond to the request.  But return once the right
		response is given.

		"""
		while 1:
			try: req_num, object = self._get()
			except ConnectionQuit:
				self._put("quitting", self.get_new_req_num())
				self._close()
				return
			if req_num == desired_req_num: return object
			else:
				assert isinstance(object, ConnectionRequest)
				self.answer_request(object, req_num)

	def answer_request(self, request, req_num):
		"""Put the object requested by request down the pipe"""
		del self.unused_request_numbers[req_num]
		argument_list = []
		for i in range(request.num_args):
			arg_req_num, arg = self._get()
			assert arg_req_num == req_num
			argument_list.append(arg)
		try:
			Security.vet_request(request, argument_list)
			result = apply(eval(request.function_string), argument_list)
		except: result = self.extract_exception()
		self._put(result, req_num)
		self.unused_request_numbers[req_num] = None

	def extract_exception(self):
		"""Return active exception"""
		if robust.is_routine_fatal(sys.exc_info()[1]):
			raise # Fatal error--No logging necessary, but connection down
		if log.Log.verbosity >= 5 or log.Log.term_verbosity >= 5:
			log.Log("Sending back exception %s of type %s: \n%s" %
					(sys.exc_info()[1], sys.exc_info()[0],
					 "".join(traceback.format_tb(sys.exc_info()[2]))), 5)
		return sys.exc_info()[1]

	def Server(self):
		"""Start server's read eval return loop"""
		Globals.server = 1
		Globals.connections.append(self)
		log.Log("Starting server", 6)
		self.get_response(-1)

	def reval(self, function_string, *args):
		"""Execute command on remote side

		The first argument should be a string that evaluates to a
		function, like "pow", and the remaining are arguments to that
		function.

		"""
		req_num = self.get_new_req_num()
		self._put(ConnectionRequest(function_string, len(args)), req_num)
		for arg in args: self._put(arg, req_num)
		result = self.get_response(req_num)
		self.unused_request_numbers[req_num] = None
		if isinstance(result, Exception): raise result
		elif isinstance(result, SystemExit): raise result
		elif isinstance(result, KeyboardInterrupt): raise result
		else: return result

	def get_new_req_num(self):
		"""Allot a new request number and return it"""
		if not self.unused_request_numbers:
			raise ConnectionError("Exhaused possible connection numbers")
		req_num = self.unused_request_numbers.keys()[0]
		del self.unused_request_numbers[req_num]
		return req_num

	def quit(self):
		"""Close the associated pipes and tell server side to quit"""
		assert not Globals.server
		self._putquit()
		self._get()
		self._close()

	def __getattr__(self, name):
		"""Intercept attributes to allow for . invocation"""
		return EmulateCallable(self, name)


class RedirectedConnection(Connection):
	"""Represent a connection more than one move away

	For instance, suppose things are connected like this: S1---C---S2.
	If Server1 wants something done by Server2, it will have to go
	through the Client.  So on S1's side, S2 will be represented by a
	RedirectedConnection.

	"""
	def __init__(self, conn_number, routing_number = 0):
		"""RedirectedConnection initializer

		Returns a RedirectedConnection object for the given
		conn_number, where commands are routed through the connection
		with the given routing_number.  0 is the client, so the
		default shouldn't have to be changed.

		"""
		self.conn_number = conn_number
		self.routing_number = routing_number
		self.routing_conn = Globals.connection_dict[routing_number]

	def reval(self, function_string, *args):
		"""Evalution function_string on args on remote connection"""
		return self.routing_conn.reval("RedirectedRun", self.conn_number,
									   function_string, *args)

	def __str__(self):
		return "RedirectedConnection %d,%d" % (self.conn_number,
											   self.routing_number)

	def __getattr__(self, name):
		return EmulateCallableRedirected(self.conn_number, self.routing_conn,
										 name)

def RedirectedRun(conn_number, func, *args):
	"""Run func with args on connection with conn number conn_number

	This function is meant to redirect requests from one connection to
	another, so conn_number must not be the local connection (and also
	for security reasons since this function is always made
	available).

	"""
	conn = Globals.connection_dict[conn_number]
	assert conn is not Globals.local_connection, conn
	return conn.reval(func, *args)


class EmulateCallable:
	"""This is used by PipeConnection in calls like conn.os.chmod(foo)"""
	def __init__(self, connection, name):
		self.connection = connection
		self.name = name
	def __call__(self, *args):
		return apply(self.connection.reval, (self.name,) + args)
	def __getattr__(self, attr_name):
		return EmulateCallable(self.connection,
							   "%s.%s" % (self.name, attr_name))

class EmulateCallableRedirected:
	"""Used by RedirectedConnection in calls like conn.os.chmod(foo)"""
	def __init__(self, conn_number, routing_conn, name):
		self.conn_number, self.routing_conn = conn_number, routing_conn
		self.name = name
	def __call__(self, *args):
		return apply(self.routing_conn.reval,
					 ("RedirectedRun", self.conn_number, self.name) + args)
	def __getattr__(self, attr_name):
		return EmulateCallableRedirected(self.conn_number, self.routing_conn,
										 "%s.%s" % (self.name, attr_name))


class VirtualFile:
	"""When the client asks for a file over the connection, it gets this

	The returned instance then forwards requests over the connection.
	The class's dictionary is used by the server to associate each
	with a unique file number.

	"""
	#### The following are used by the server
	vfiles = {}
	counter = 0

	def getbyid(cls, id):
		return cls.vfiles[id]
	getbyid = classmethod(getbyid)

	def readfromid(cls, id, length):
		if length is None: return cls.vfiles[id].read()
		else: return cls.vfiles[id].read(length)
	readfromid = classmethod(readfromid)

	def readlinefromid(cls, id):
		return cls.vfiles[id].readline()
	readlinefromid = classmethod(readlinefromid)

	def writetoid(cls, id, buffer):
		return cls.vfiles[id].write(buffer)
	writetoid = classmethod(writetoid)

	def closebyid(cls, id):
		fp = cls.vfiles[id]
		del cls.vfiles[id]
		return fp.close()
	closebyid = classmethod(closebyid)

	def new(cls, fileobj):
		"""Associate a new VirtualFile with a read fileobject, return id"""
		count = cls.counter
		cls.vfiles[count] = fileobj
		cls.counter = count + 1
		return count
	new = classmethod(new)


	#### And these are used by the client
	def __init__(self, connection, id):
		self.connection = connection
		self.id = id

	def read(self, length = None):
		return self.connection.VirtualFile.readfromid(self.id, length)

	def readline(self):
		return self.connection.VirtualFile.readlinefromid(self.id)

	def write(self, buf):
		return self.connection.VirtualFile.writetoid(self.id, buf)

	def close(self):
		return self.connection.VirtualFile.closebyid(self.id)

	def __iter__(self):
		"""Iterates lines in file, like normal iter(file) behavior"""
		while 1:
			line = self.readline()
			if not line: break
			yield line


# everything has to be available here for remote connection's use, but
# put at bottom to reduce circularities.
import Globals, Time, Rdiff, Hardlink, FilenameMapping, C, Security, \
	   Main, rorpiter, selection, increment, statistics, manage, lazy, \
	   iterfile, rpath, robust, restore, manage, backup, connection, \
	   TempFile, SetConnections, librsync, log, regress, fs_abilities, \
	   eas_acls, user_group, compare

try: import win_acls
except ImportError: pass

Globals.local_connection = LocalConnection()
Globals.connections.append(Globals.local_connection)
# Following changed by server in SetConnections
Globals.connection_dict[0] = Globals.local_connection




# Copyright 2002, 2003, 2004, 2005 Ben Escoto
#
# This file is part of rdiff-backup.
#
# rdiff-backup is free software; you can redistribute it and/or modify
# under the terms of the GNU General Public License as published by the
# Free Software Foundation; either version 2 of the License, or (at your
# option) any later version.
#
# rdiff-backup is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with rdiff-backup; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
# USA

"""Start (and end) here - read arguments, set global settings, etc."""

from __future__ import generators
import getopt, sys, re, os, cStringIO, tempfile
from log import Log, LoggerError, ErrorLog
import Globals, Time, SetConnections, selection, robust, rpath, \
	   manage, backup, connection, restore, FilenameMapping, \
	   Security, Hardlink, regress, C, fs_abilities, statistics, compare


action = None
create_full_path = None
remote_cmd, remote_schema = None, None
force = None
select_opts = []
select_files = []
user_mapping_filename, group_mapping_filename, preserve_numerical_ids = \
					   None, None, None

# These are global because they are set while we are trying to figure
# whether to restore or to backup
restore_root, restore_index, restore_root_set = None, None, 0
return_val = None # Set to cause exit code to be specified value

def parse_cmdlineoptions(arglist):
	"""Parse argument list and set global preferences"""
	global args, action, create_full_path, force, restore_timestr, remote_cmd
	global remote_schema, remove_older_than_string
	global user_mapping_filename, group_mapping_filename, \
		   preserve_numerical_ids

	def sel_fl(filename):
		"""Helper function for including/excluding filelists below"""
		try: return open(filename, "r")
		except IOError: Log.FatalError("Error opening file %s" % filename)

	def normalize_path(path):
		"""Used below to normalize the security paths before setting"""
		return rpath.RPath(Globals.local_connection, path).normalize().path

	try: optlist, args = getopt.getopt(arglist, "blr:sv:V",
		 ["backup-mode", "calculate-average", "carbonfile",
		  "check-destination-dir",
		  "compare", "compare-at-time=", "compare-hash",
		  "compare-hash-at-time=", "compare-full", "compare-full-at-time=",
		  "create-full-path", "current-time=", "exclude=",
		  "exclude-device-files", "exclude-fifos", "exclude-filelist=",
		  "exclude-symbolic-links", "exclude-sockets",
		  "exclude-filelist-stdin", "exclude-globbing-filelist=",
		  "exclude-globbing-filelist-stdin", "exclude-mirror=",
		  "exclude-other-filesystems", "exclude-regexp=", "exclude-if-present=",
		  "exclude-special-files", "force", "group-mapping-file=",
		  "include=", "include-filelist=", "include-filelist-stdin",
		  "include-globbing-filelist=",
		  "include-globbing-filelist-stdin", "include-regexp=",
		  "include-special-files", "include-symbolic-links",
		  "list-at-time=", "list-changed-since=", "list-increments",
		  "list-increment-sizes", "never-drop-acls",
		  "max-file-size=", "min-file-size=",
		  "no-acls", "no-carbonfile",
		  "no-compare-inode", "no-compression", "no-compression-regexp=",
		  "no-eas", "no-file-statistics", "no-hard-links", "null-separator",
		  "override-chars-to-quote=", "parsable-output",
		  "preserve-numerical-ids", "print-statistics",
		  "remote-cmd=", "remote-schema=", "remote-tempdir=",
		  "remove-older-than=", "restore-as-of=", "restrict=",
		  "restrict-read-only=", "restrict-update-only=", "server",
		  "ssh-no-compression", "tempdir=", "terminal-verbosity=",
		  "test-server", "user-mapping-file=", "verbosity=", "verify",
		  "verify-at-time=", "version"])
	except getopt.error, e:
		commandline_error("Bad commandline options: " + str(e))

	for opt, arg in optlist:
		if opt == "-b" or opt == "--backup-mode": action = "backup"
		elif opt == "--calculate-average": action = "calculate-average"
		elif opt == "--carbonfile": Globals.set("carbonfile_active", 1)
		elif opt == "--check-destination-dir": action = "check-destination-dir"
		elif opt in ("--compare", "--compare-at-time",
					 "--compare-hash", "--compare-hash-at-time",
					 "--compare-full", "--compare-full-at-time"):
			if opt[-8:] == "-at-time": restore_timestr, opt = arg, opt[:-8]
			else: restore_timestr = "now"
			action = opt[2:]
		elif opt == "--create-full-path": create_full_path = 1
		elif opt == "--current-time":
			Globals.set_integer('current_time', arg)
		elif (opt == "--exclude" or
			  opt == "--exclude-device-files" or
			  opt == "--exclude-fifos" or
			  opt == "--exclude-other-filesystems" or
			  opt == "--exclude-regexp" or
			  opt == "--exclude-if-present" or
			  opt == "--exclude-special-files" or
			  opt == "--exclude-sockets" or
			  opt == "--exclude-symbolic-links"):
			select_opts.append((opt, arg))
		elif opt == "--exclude-filelist":
			select_opts.append((opt, arg))
			select_files.append(sel_fl(arg))
		elif opt == "--exclude-filelist-stdin":
			select_opts.append(("--exclude-filelist", "standard input"))
			select_files.append(sys.stdin)
		elif opt == "--exclude-globbing-filelist":
			select_opts.append((opt, arg))
			select_files.append(sel_fl(arg))
		elif opt == "--exclude-globbing-filelist-stdin":
			select_opts.append(("--exclude-globbing-filelist",
								"standard input"))
			select_files.append(sys.stdin)
		elif opt == "--force": force = 1
		elif opt == "--group-mapping-file": group_mapping_filename = arg
		elif (opt == "--include" or
			  opt == "--include-special-files" or
			  opt == "--include-symbolic-links"):
			select_opts.append((opt, arg))
		elif opt == "--include-filelist":
			select_opts.append((opt, arg))
			select_files.append(sel_fl(arg))
		elif opt == "--include-filelist-stdin":
			select_opts.append(("--include-filelist", "standard input"))
			select_files.append(sys.stdin)
		elif opt == "--include-globbing-filelist":
			select_opts.append((opt, arg))
			select_files.append(sel_fl(arg))
		elif opt == "--include-globbing-filelist-stdin":
			select_opts.append(("--include-globbing-filelist",
								"standard input"))
			select_files.append(sys.stdin)
		elif opt == "--include-regexp": select_opts.append((opt, arg))
		elif opt == "--list-at-time":
			restore_timestr, action = arg, "list-at-time"
		elif opt == "--list-changed-since":
			restore_timestr, action = arg, "list-changed-since"
		elif opt == "-l" or opt == "--list-increments":
			action = "list-increments"
		elif opt == '--list-increment-sizes': action = 'list-increment-sizes'
		elif opt == "--max-file-size": select_opts.append((opt, arg))
		elif opt == "--min-file-size": select_opts.append((opt, arg))
		elif opt == "--never-drop-acls": Globals.set("never_drop_acls", 1)
		elif opt == "--no-acls":
			Globals.set("acls_active", 0)
			Globals.set("win_acls_active", 0)
		elif opt == "--no-carbonfile": Globals.set("carbonfile_active", 0)
		elif opt == "--no-compare-inode": Globals.set("compare_inode", 0)
		elif opt == "--no-compression": Globals.set("compression", None)
		elif opt == "--no-compression-regexp":
			Globals.set("no_compression_regexp_string", arg)
		elif opt == "--no-eas": Globals.set("eas_active", 0)
		elif opt == "--no-file-statistics": Globals.set('file_statistics', 0)
		elif opt == "--no-hard-links": Globals.set('preserve_hardlinks', 0)
		elif opt == "--null-separator": Globals.set("null_separator", 1)
		elif opt == "--override-chars-to-quote":
			Globals.set('chars_to_quote', arg)
		elif opt == "--parsable-output": Globals.set('parsable_output', 1)
		elif opt == "--preserve-numerical-ids": preserve_numerical_ids = 1
		elif opt == "--print-statistics": Globals.set('print_statistics', 1)
		elif opt == "-r" or opt == "--restore-as-of":
			restore_timestr, action = arg, "restore-as-of"
		elif opt == "--remote-cmd": remote_cmd = arg
		elif opt == "--remote-schema": remote_schema = arg
		elif opt == "--remote-tempdir": Globals.remote_tempdir = arg
		elif opt == "--remove-older-than":
			remove_older_than_string = arg
			action = "remove-older-than"
		elif opt == "--no-resource-forks":
			Globals.set('resource_forks_active', 0)
		elif opt == "--restrict": Globals.restrict_path = normalize_path(arg)
		elif opt == "--restrict-read-only":
			Globals.security_level = "read-only"
			Globals.restrict_path = normalize_path(arg)
		elif opt == "--restrict-update-only":
			Globals.security_level = "update-only"
			Globals.restrict_path = normalize_path(arg)
		elif opt == "-s" or opt == "--server":
			action = "server"
			Globals.server = 1
		elif opt == "--ssh-no-compression":
			Globals.set('ssh_compression', None)
		elif opt == "--tempdir": tempfile.tempdir = arg
		elif opt == "--terminal-verbosity": Log.setterm_verbosity(arg)
		elif opt == "--test-server": action = "test-server"
		elif opt == "--user-mapping-file": user_mapping_filename = arg
		elif opt == "-v" or opt == "--verbosity": Log.setverbosity(arg)
		elif opt == "--verify": action, restore_timestr = "verify", "now"
		elif opt == "--verify-at-time": action, restore_timestr = "verify", arg
		elif opt == "-V" or opt == "--version":
			print "rdiff-backup " + Globals.version
			sys.exit(0)
		else: Log.FatalError("Unknown option %s" % opt)
	Log("Using rdiff-backup version %s" % (Globals.version), 4)

def check_action():
	"""Check to make sure action is compatible with args"""
	global action
	arg_action_dict = {0: ['server'],
					   1: ['list-increments', 'list-increment-sizes',
						   'remove-older-than', 'list-at-time',
						   'list-changed-since', 'check-destination-dir',
						   'verify'],
					   2: ['backup', 'restore', 'restore-as-of',
						   'compare', 'compare-hash', 'compare-full']}
	l = len(args)
	if l == 0 and action not in arg_action_dict[l]:
		commandline_error("No arguments given")
	elif not action:
		if l == 2: pass # Will determine restore or backup later
		else:
			commandline_error("Switches missing or wrong number of arguments")
	elif action == 'test-server' or action == 'calculate-average':
		pass # these two take any number of args
	elif l > 2 or action not in arg_action_dict[l]:
		commandline_error("Wrong number of arguments given.")

def final_set_action(rps):
	"""If no action set, decide between backup and restore at this point"""
	global action
	if action: return
	assert len(rps) == 2, rps
	if restore_set_root(rps[0]): action = "restore"
	else: action = "backup"

def commandline_error(message):
	Log.FatalError(message + "\nSee the rdiff-backup manual page for "
				   "more information.")

def misc_setup(rps):
	"""Set default change ownership flag, umask, relay regexps"""
	os.umask(077)
	Time.setcurtime(Globals.current_time)
	SetConnections.UpdateGlobal("client_conn", Globals.local_connection)
	Globals.postset_regexp('no_compression_regexp',
						   Globals.no_compression_regexp_string)
	for conn in Globals.connections:
		conn.robust.install_signal_handlers()
		conn.Hardlink.initialize_dictionaries()

def init_user_group_mapping(destination_conn):
	"""Initialize user and group mapping on destination connection"""
	global user_mapping_filename, group_mapping_filename, \
		   preserve_numerical_ids
	def get_string_from_file(filename):
		if not filename: return None
		rp = rpath.RPath(Globals.local_connection, filename)
		try: return rp.get_data()
		except OSError, e:
			Log.FatalError("Error '%s' reading mapping file '%s'" %
						   (str(e), filename))
	user_mapping_string = get_string_from_file(user_mapping_filename)
	destination_conn.user_group.init_user_mapping(user_mapping_string,
												  preserve_numerical_ids)
	group_mapping_string = get_string_from_file(group_mapping_filename)
	destination_conn.user_group.init_group_mapping(group_mapping_string,
												   preserve_numerical_ids)

def take_action(rps):
	"""Do whatever action says"""
	if action == "server":
		connection.PipeConnection(sys.stdin, sys.stdout).Server()
		sys.exit(0)
	elif action == "backup": Backup(rps[0], rps[1])
	elif action == "calculate-average": CalculateAverage(rps)
	elif action == "check-destination-dir": CheckDest(rps[0])
	elif action.startswith("compare"): Compare(action, rps[0], rps[1])
	elif action == "list-at-time": ListAtTime(rps[0])
	elif action == "list-changed-since": ListChangedSince(rps[0])
	elif action == "list-increments": ListIncrements(rps[0])
	elif action == 'list-increment-sizes': ListIncrementSizes(rps[0])
	elif action == "remove-older-than": RemoveOlderThan(rps[0])
	elif action == "restore": Restore(*rps)
	elif action == "restore-as-of": Restore(rps[0], rps[1], 1)
	elif action == "test-server": SetConnections.TestConnections()
	elif action == "verify": Verify(rps[0])
	else: raise AssertionError("Unknown action " + action)

def cleanup():
	"""Do any last minute cleaning before exiting"""
	Log("Cleaning up", 6)
	if ErrorLog.isopen(): ErrorLog.close()
	Log.close_logfile()
	if not Globals.server: SetConnections.CloseConnections()

def error_check_Main(arglist):
	"""Run Main on arglist, suppressing stack trace for routine errors"""
	try: Main(arglist)
	except SystemExit: raise
	except (Exception, KeyboardInterrupt), exc:
		errmsg = robust.is_routine_fatal(exc)
		if errmsg:
			Log.exception(2, 6)
			Log.FatalError(errmsg)
		else:
			Log.exception(2, 2)
			raise

def Main(arglist):
	"""Start everything up!"""
	parse_cmdlineoptions(arglist)
	check_action()
	cmdpairs = SetConnections.get_cmd_pairs(args, remote_schema, remote_cmd)
	Security.initialize(action or "mirror", cmdpairs)
	rps = map(SetConnections.cmdpair2rp, cmdpairs)
	final_set_action(rps)
	misc_setup(rps)
	take_action(rps)
	cleanup()
	if return_val is not None: sys.exit(return_val)

def Backup(rpin, rpout):
	"""Backup, possibly incrementally, src_path to dest_path."""
	global incdir
	SetConnections.BackupInitConnections(rpin.conn, rpout.conn)
	backup_check_dirs(rpin, rpout)
	backup_set_rbdir(rpin, rpout)
	rpout.conn.fs_abilities.backup_set_globals(rpin, force)
	if Globals.chars_to_quote: rpout = backup_quoted_rpaths(rpout)
	init_user_group_mapping(rpout.conn)
	backup_final_init(rpout)
	backup_set_select(rpin)
	backup_warn_if_infinite_regress(rpin, rpout)
	if prevtime:
		Time.setprevtime(prevtime)
		rpout.conn.Main.backup_touch_curmirror_local(rpin, rpout)
		backup.Mirror_and_increment(rpin, rpout, incdir)
		rpout.conn.Main.backup_remove_curmirror_local()
	else:
		backup.Mirror(rpin, rpout)
		rpout.conn.Main.backup_touch_curmirror_local(rpin, rpout)

def backup_quoted_rpaths(rpout):
	"""Get QuotedRPath versions of important RPaths.  Return rpout"""
	global incdir
	SetConnections.UpdateGlobal(
		'rbdir', FilenameMapping.get_quotedrpath(Globals.rbdir))
	incdir = FilenameMapping.get_quotedrpath(incdir)
	return FilenameMapping.get_quotedrpath(rpout)

def backup_set_select(rpin):
	"""Create Select objects on source connection"""
	if rpin.conn.os.name == 'nt':
		Log("Symbolic links excluded by default on Windows", 4)
		select_opts.append(("--exclude-symbolic-links", None))
	rpin.conn.backup.SourceStruct.set_source_select(rpin, select_opts,
													*select_files)

def backup_check_dirs(rpin, rpout):
	"""Make sure in and out dirs exist and are directories"""
	if rpout.lstat() and not rpout.isdir():
		if not force: Log.FatalError("Destination %s exists and is not a "
									 "directory" % rpout.path)
		else:
			Log("Deleting %s" % rpout.path, 3)
			rpout.delete()
	if not rpout.lstat():
		try: 
			if create_full_path: rpout.makedirs()
			else: rpout.mkdir()
		except os.error:
			Log.FatalError("Unable to create directory %s" % rpout.path)

	if not rpin.lstat():
		Log.FatalError("Source directory %s does not exist" % rpin.path)
	elif not rpin.isdir():
		Log.FatalError("Source %s is not a directory" % rpin.path)
	Globals.rbdir = rpout.append_path("rdiff-backup-data")

def check_failed_initial_backup():
	"""Returns true if it looks like initial backup failed."""
	if Globals.rbdir.lstat():
		rbdir_files = Globals.rbdir.listdir()
		mirror_markers = filter(lambda x: x.startswith("current_mirror"),
								rbdir_files)
		error_logs = filter(lambda x: x.startswith("error_log"),
							rbdir_files)
		metadata_mirrors = filter(lambda x: x.startswith("mirror_metadata"),
								rbdir_files)
		# If we have no current_mirror marker, and the increments directory
		# is empty, we most likely have a failed backup.
		return not mirror_markers and len(error_logs) <= 1 and \
				len(metadata_mirrors) <= 1
	return False

def fix_failed_initial_backup():
	"""Clear Globals.rbdir after a failed initial backup"""
	Log("Found interrupted initial backup. Removing...",  2)
	rbdir_files = Globals.rbdir.listdir()
	# Try to delete the increments dir first
	if 'increments' in rbdir_files:
		rbdir_files.remove('increments')
		rp = Globals.rbdir.append('increments')
		try:
			rp.conn.rpath.delete_dir_no_files(rp)
		except rpath.RPathException:
			Log("Increments dir contains files.", 4)
			return
		except Security.Violation:
			Log("Server doesn't support resuming.", 2)
			return
	
	for file_name in rbdir_files:
		rp = Globals.rbdir.append_path(file_name)
		if not rp.isdir(): # Only remove files, not folders
			rp.delete()

def backup_set_rbdir(rpin, rpout):
	"""Initialize data dir and logging"""
	global incdir
	try:
		incdir = Globals.rbdir.append_path("increments")
	except (OSError, IOError), exc:
		Log.FatalError("Could not begin backup due to\n%s" % exc)

	assert rpout.lstat(), (rpout.path, rpout.lstat())
	if rpout.isdir() and not rpout.listdir(): # rpout is empty dir
		try:
			rpout.chmod(0700) # just make sure permissions aren't too lax
		except OSError:
			Log("Cannot change permissions on target directory.", 2)
	elif not Globals.rbdir.lstat() and not force: Log.FatalError(
"""Destination directory

%s

exists, but does not look like a rdiff-backup directory.  Running
rdiff-backup like this could mess up what is currently in it.  If you
want to update or overwrite it, run rdiff-backup with the --force
option.""" % rpout.path)
	elif check_failed_initial_backup():
		fix_failed_initial_backup()

	if not Globals.rbdir.lstat():
		try:
			Globals.rbdir.mkdir()
		except (OSError, IOError), exc:
			Log.FatalError(
"""Could not create rdiff-backup directory

%s

due to

%s

Please check that the rdiff-backup user can create files and directories in the
destination directory: %s""" % (Globals.rbdir.path, exc, rpout.path))
	SetConnections.UpdateGlobal('rbdir', Globals.rbdir)

def backup_warn_if_infinite_regress(rpin, rpout):
	"""Warn user if destination area contained in source area"""
	# Just a few heuristics, we don't have to get every case
	if rpout.conn is not rpin.conn: return
	if len(rpout.path) <= len(rpin.path)+1: return
	if rpout.path[:len(rpin.path)+1] != rpin.path + '/': return

	relative_rpout_comps = tuple(rpout.path[len(rpin.path)+1:].split('/'))
	relative_rpout = rpin.new_index(relative_rpout_comps)
	if not Globals.select_mirror.Select(relative_rpout): return

	Log(
"""Warning: The destination directory '%s' may be contained in the
source directory '%s'.  This could cause an infinite regress.  You
may need to use the --exclude option.""" % (rpout.path, rpin.path), 2)

def backup_get_mirrortime():
	"""Return time in seconds of previous mirror, or None if cannot"""
	incbase = Globals.rbdir.append_path("current_mirror")
	mirror_rps = restore.get_inclist(incbase)
	assert len(mirror_rps) <= 1, \
		   "Found %s current_mirror rps, expected <=1" % (len(mirror_rps),)
	if mirror_rps: return mirror_rps[0].getinctime()
	else: return None

def backup_final_init(rpout):
	"""Open the backup log and the error log, create increments dir"""
	global prevtime, incdir
	if Log.verbosity > 0:
		Log.open_logfile(Globals.rbdir.append("backup.log"))
	checkdest_if_necessary(rpout)
	prevtime = backup_get_mirrortime()
	if prevtime >= Time.curtime: Log.FatalError(
"""Time of Last backup is not in the past.  This is probably caused
by running two backups in less than a second.  Wait a second a try again.""")
	ErrorLog.open(Time.curtimestr, compress = Globals.compression)
	if not incdir.lstat(): incdir.mkdir()

def backup_touch_curmirror_local(rpin, rpout):
	"""Make a file like current_mirror.time.data to record time

	When doing an incremental backup, this should happen before any
	other writes, and the file should be removed after all writes.
	That way we can tell whether the previous session aborted if there
	are two current_mirror files.

	When doing the initial full backup, the file can be created after
	everything else is in place.

	"""
	mirrorrp = Globals.rbdir.append("current_mirror.%s.%s" % (Time.curtimestr,
															  "data"))
	Log("Writing mirror marker %s" % mirrorrp.path, 6)
	try: pid = os.getpid()
	except: pid = "NA"
	mirrorrp.write_string("PID %s\n" % (pid,))
	mirrorrp.fsync_with_dir()

def backup_remove_curmirror_local():
	"""Remove the older of the current_mirror files.  Use at end of session"""
	assert Globals.rbdir.conn is Globals.local_connection
	curmir_incs = restore.get_inclist(Globals.rbdir.append("current_mirror"))
	assert len(curmir_incs) == 2
	if curmir_incs[0].getinctime() < curmir_incs[1].getinctime():
		older_inc = curmir_incs[0]
	else: older_inc = curmir_incs[1]

	C.sync() # Make sure everything is written before curmirror is removed
	older_inc.delete()


def Restore(src_rp, dest_rp, restore_as_of = None):
	"""Main restoring function

	Here src_rp should be the source file (either an increment or
	mirror file), dest_rp should be the target rp to be written.

	"""
	if not restore_root_set and not restore_set_root(src_rp):
		Log.FatalError("Could not find rdiff-backup repository at "
					   + src_rp.path)
	restore_check_paths(src_rp, dest_rp, restore_as_of)
	try:
		dest_rp.conn.fs_abilities.restore_set_globals(dest_rp)
	except (OSError, IOError), exc:
		print "\n"
		Log.FatalError("Could not begin backup due to\n%s" % exc)
	init_user_group_mapping(dest_rp.conn)
	src_rp = restore_init_quoting(src_rp)
	restore_check_backup_dir(restore_root, src_rp, restore_as_of)
	inc_rpath = Globals.rbdir.append_path('increments', restore_index)
	if restore_as_of:
		try: time = Time.genstrtotime(restore_timestr, rp = inc_rpath)
		except Time.TimeException, exc: Log.FatalError(str(exc))
	else: time = src_rp.getinctime()
	restore_set_select(restore_root, dest_rp)
	restore_start_log(src_rp, dest_rp, time)
	restore.Restore(restore_root.new_index(restore_index),
					inc_rpath, dest_rp, time)
	Log("Restore finished", 4)

def restore_init_quoting(src_rp):
	"""Change rpaths into quoted versions of themselves if necessary"""
	global restore_root
	if not Globals.chars_to_quote: return src_rp
	for conn in Globals.connections: conn.FilenameMapping.set_init_quote_vals()
	restore_root = FilenameMapping.get_quotedrpath(restore_root)
	SetConnections.UpdateGlobal(
		'rbdir', FilenameMapping.get_quotedrpath(Globals.rbdir))
	return FilenameMapping.get_quotedrpath(src_rp)

def restore_set_select(mirror_rp, target):
	"""Set the selection iterator on both side from command line args

	We must set both sides because restore filtering is different from
	select filtering.  For instance, if a file is excluded it should
	not be deleted from the target directory.

	The StringIO stuff is because filelists need to be read and then
	duplicated, because we need two copies of them now.

	"""
	def fp2string(fp):
		buf = fp.read()
		assert not fp.close()
		return buf
	select_data = map(fp2string, select_files)
	if select_opts: 
		mirror_rp.conn.restore.MirrorStruct.set_mirror_select(
			target, select_opts, *map(cStringIO.StringIO, select_data))
		target.conn.restore.TargetStruct.set_target_select(
			target, select_opts, *map(cStringIO.StringIO, select_data))

def restore_start_log(rpin, target, time):
	"""Open restore log file, log initial message"""
	try: Log.open_logfile(Globals.rbdir.append("restore.log"))
	except (LoggerError, Security.Violation), e:
		Log("Warning - Unable to open logfile: " + str(e), 2)

	# Log following message at file verbosity 3, but term verbosity 4
	log_message = ("Starting restore of %s to %s as it was as of %s." %
				   (rpin.path, target.path, Time.timetopretty(time)))
	if Log.term_verbosity >= 4: Log.log_to_term(log_message, 4)
	if Log.verbosity >= 3: Log.log_to_file(log_message)

def restore_check_paths(rpin, rpout, restoreasof = None):
	"""Make sure source and destination exist, and have appropriate type"""
	if not restoreasof:
		if not rpin.lstat():
			Log.FatalError("Source file %s does not exist" % rpin.path)
	if not force and rpout.lstat() and (not rpout.isdir() or rpout.listdir()):
		Log.FatalError("Restore target %s already exists, "
					   "specify --force to overwrite." % rpout.path)
	if force and rpout.lstat() and not rpout.isdir(): rpout.delete()

def restore_check_backup_dir(mirror_root, src_rp = None, restore_as_of = 1):
	"""Make sure backup dir root rpin is in consistent state"""
	if not restore_as_of and not src_rp.isincfile():
		Log.FatalError("""File %s does not look like an increment file.

Try restoring from an increment file (the filenames look like
"foobar.2001-09-01T04:49:04-07:00.diff").""" % src_rp.path)

	result = checkdest_need_check(mirror_root)
	if result is None:
		Log.FatalError("%s does not appear to be an rdiff-backup directory."
					   % (Globals.rbdir.path,))
	elif result == 1: Log.FatalError(
		"Previous backup to %s seems to have failed.\nRerun rdiff-backup "
		"with --check-destination-dir option to revert directory "
		"to state before unsuccessful session." % (mirror_root.path,))

def restore_set_root(rpin):
	"""Set data dir, restore_root and index, or return None if fail

	The idea here is to keep backing up on the path until we find
	a directory that contains "rdiff-backup-data".  That is the
	mirror root.  If the path from there starts
	"rdiff-backup-data/increments*", then the index is the
	remainder minus that.  Otherwise the index is just the path
	minus the root.

	All this could fail if the increment file is pointed to in a
	funny way, using symlinks or somesuch.

	"""
	global restore_root, restore_index, restore_root_set
	if rpin.isincfile(): relpath = rpin.getincbase().path
	else: relpath = rpin.path
	if rpin.conn is not Globals.local_connection:
		# For security checking consistency, don't get absolute path
		pathcomps = relpath.split('/')
	else: pathcomps = os.path.join(os.getcwd(), relpath).split("/")
	if not pathcomps[0]: min_len_pathcomps = 2 # treat abs paths differently
	else: min_len_pathcomps = 1

	i = len(pathcomps)
	while i >= min_len_pathcomps:
		parent_dir = rpath.RPath(rpin.conn, "/".join(pathcomps[:i]))
		if (parent_dir.isdir() and parent_dir.readable() and
			"rdiff-backup-data" in parent_dir.listdir()): break
		if parent_dir.path == rpin.conn.Globals.get('restrict_path'):
			return None
		i = i-1
	else: return None

	restore_root = parent_dir
	Log("Using mirror root directory %s" % restore_root.path, 6)
	if restore_root.conn is Globals.local_connection:
		Security.reset_restrict_path(restore_root)
	SetConnections.UpdateGlobal('rbdir',
								restore_root.append_path("rdiff-backup-data"))
	if not Globals.rbdir.isdir():
		Log.FatalError("Unable to read rdiff-backup-data directory %s" %
					   Globals.rbdir.path)

	from_datadir = tuple(pathcomps[i:])
	if not from_datadir or from_datadir[0] != "rdiff-backup-data":
		restore_index = from_datadir # in mirror, not increments
	else:
		assert (from_datadir[1] == "increments" or
				(len(from_datadir) == 2 and
				 from_datadir[1].startswith('increments'))), from_datadir
		restore_index = from_datadir[2:]
	restore_root_set = 1
	return 1


def ListIncrements(rp):
	"""Print out a summary of the increments and their times"""
	rp = require_root_set(rp, 1)
	restore_check_backup_dir(restore_root)
	mirror_rp = restore_root.new_index(restore_index)
	inc_rpath = Globals.rbdir.append_path('increments', restore_index)
	incs = restore.get_inclist(inc_rpath)
	mirror_time = restore.MirrorStruct.get_mirror_time()
	if Globals.parsable_output:
		print manage.describe_incs_parsable(incs, mirror_time, mirror_rp)
	else: print manage.describe_incs_human(incs, mirror_time, mirror_rp)

def require_root_set(rp, read_only):
	"""Make sure rp is or is in a valid rdiff-backup dest directory.

	Also initializes fs_abilities (read or read/write) and quoting and
	return quoted rp if necessary.

	"""
	if not restore_set_root(rp):
		Log.FatalError(("Bad directory %s.\n" % (rp.path,)) +
		  "It doesn't appear to be an rdiff-backup destination dir")
	try:
		Globals.rbdir.conn.fs_abilities.single_set_globals(Globals.rbdir,
														   read_only)
	except (OSError, IOError), exc:
		print("\n")
		Log.FatalError("Could not open rdiff-backup directory\n\n%s\n\n"
					   "due to\n\n%s" % (Globals.rbdir.path, exc))
	if Globals.chars_to_quote: return restore_init_quoting(rp)
	else: return rp
	

def ListIncrementSizes(rp):
	"""Print out a summary of the increments """
	rp = require_root_set(rp, 1)
	print manage.ListIncrementSizes(restore_root, restore_index)


def CalculateAverage(rps):
	"""Print out the average of the given statistics files"""
	statobjs = map(lambda rp: statistics.StatsObj().read_stats_from_rp(rp),
				   rps)
	average_stats = statistics.StatsObj().set_to_average(statobjs)
	print average_stats.get_stats_logstring(
		"Average of %d stat files" % len(rps))


def RemoveOlderThan(rootrp):
	"""Remove all increment files older than a certain time"""
	rootrp = require_root_set(rootrp, 0)
	rot_require_rbdir_base(rootrp)

	time = rot_check_time(remove_older_than_string)
	if time is None: return
	Log("Actual remove older than time: %s" % (time,), 6)
	manage.delete_earlier_than(Globals.rbdir, time)

def rot_check_time(time_string):
	"""Check remove older than time_string, return time in seconds"""
	try: time = Time.genstrtotime(time_string)
	except Time.TimeException, exc: Log.FatalError(str(exc))

	times_in_secs = [inc.getinctime() for inc in 
		  restore.get_inclist(Globals.rbdir.append_path("increments"))]
	times_in_secs = filter(lambda t: t < time, times_in_secs)
	if not times_in_secs:
		Log("No increments older than %s found, exiting." %
			(Time.timetopretty(time),), 3)
		return None

	times_in_secs.sort()
	inc_pretty_time = "\n".join(map(Time.timetopretty, times_in_secs))
	if len(times_in_secs) > 1 and not force:
		Log.FatalError("Found %d relevant increments, dated:\n%s"
			"\nIf you want to delete multiple increments in this way, "
			"use the --force." % (len(times_in_secs), inc_pretty_time))
	if len(times_in_secs) == 1:
		Log("Deleting increment at time:\n" + inc_pretty_time, 3)
	else: Log("Deleting increments at times:\n" + inc_pretty_time, 3)
	return times_in_secs[-1]+1 # make sure we don't delete current increment

def rot_require_rbdir_base(rootrp):
	"""Make sure pointing to base of rdiff-backup dir"""
	if restore_index != ():
		Log.FatalError("Increments for directory %s cannot be removed "
					   "separately.\nInstead run on entire directory %s." %
					   (rootrp.path, restore_root.path))


def ListChangedSince(rp):
	"""List all the files under rp that have changed since restoretime"""
	rp = require_root_set(rp, 1)
	try: rest_time = Time.genstrtotime(restore_timestr)
	except Time.TimeException, exc: Log.FatalError(str(exc))
	mirror_rp = restore_root.new_index(restore_index)
	inc_rp = mirror_rp.append_path("increments", restore_index)
	for rorp in rp.conn.restore.ListChangedSince(mirror_rp, inc_rp, rest_time):
		# This is a hack, see restore.ListChangedSince for rationale
		print rorp.index[0]


def ListAtTime(rp):
	"""List files in archive under rp that are present at restoretime"""
	rp = require_root_set(rp, 1)
	try: rest_time = Time.genstrtotime(restore_timestr)
	except Time.TimeException, exc: Log.FatalError(str(exc))
	mirror_rp = restore_root.new_index(restore_index)
	inc_rp = mirror_rp.append_path("increments", restore_index)
	for rorp in rp.conn.restore.ListAtTime(mirror_rp, inc_rp, rest_time):
		print rorp.get_indexpath()
	

def Compare(compare_type, src_rp, dest_rp, compare_time = None):
	"""Compare metadata in src_rp with metadata of backup session

	Prints to stdout whenever a file in the src_rp directory has
	different metadata than what is recorded in the metadata for the
	appropriate session.

	Session time is read from restore_timestr if compare_time is None.

	"""
	global return_val
	dest_rp = require_root_set(dest_rp, 1)
	if not compare_time:
		try: compare_time = Time.genstrtotime(restore_timestr)
		except Time.TimeException, exc: Log.FatalError(str(exc))

	mirror_rp = restore_root.new_index(restore_index)
	inc_rp = Globals.rbdir.append_path("increments", restore_index)
	backup_set_select(src_rp) # Sets source rorp iterator
	if compare_type == "compare": compare_func = compare.Compare
	elif compare_type == "compare-hash": compare_func = compare.Compare_hash
	else:
		assert compare_type == "compare-full", compare_type
		compare_func = compare.Compare_full
	return_val = compare_func(src_rp, mirror_rp, inc_rp, compare_time)

def Verify(dest_rp, verify_time = None):
	"""Check the hashes of the regular files against mirror_metadata"""
	global return_val
	dest_rp = require_root_set(dest_rp, 1)
	if not verify_time:
		try: verify_time = Time.genstrtotime(restore_timestr)
		except Time.TimeException, exc: Log.FatalError(str(exc))

	mirror_rp = restore_root.new_index(restore_index)
	inc_rp = Globals.rbdir.append_path("increments", restore_index)
	return_val = dest_rp.conn.compare.Verify(mirror_rp, inc_rp, verify_time)


def CheckDest(dest_rp):
	"""Check the destination directory, """
	dest_rp = require_root_set(dest_rp, 0)
	need_check = checkdest_need_check(dest_rp)
	if need_check is None:
		Log.FatalError("No destination dir found at %s" % (dest_rp.path,))
	elif need_check == 0:
		Log.FatalError("Destination dir %s does not need checking" %
					   (dest_rp.path,), errlevel = 0)
	init_user_group_mapping(dest_rp.conn)
	dest_rp.conn.regress.Regress(dest_rp)

def checkdest_need_check(dest_rp):
	"""Return None if no dest dir found, 1 if dest dir needs check, 0 o/w"""
	if not dest_rp.isdir() or not Globals.rbdir.isdir(): return None
	for filename in Globals.rbdir.listdir():
		if filename not in ['chars_to_quote', 'backup.log']: break
	else: # This may happen the first backup just after we test for quoting
		return None
	curmirroot = Globals.rbdir.append("current_mirror")
	curmir_incs = restore.get_inclist(curmirroot)
	if not curmir_incs:
		Log.FatalError(
"""Bad rdiff-backup-data dir on destination side

The rdiff-backup data directory
%s
exists, but we cannot find a valid current_mirror marker.  You can
avoid this message by removing the rdiff-backup-data directory;
however any data in it will be lost.

Probably this error was caused because the first rdiff-backup session
into a new directory failed.  If this is the case it is safe to delete
the rdiff-backup-data directory because there is no important
information in it.

""" % (Globals.rbdir.path,))
	elif len(curmir_incs) == 1: return 0
	else:
		if not force:
			try:
				curmir_incs[0].conn.regress.check_pids(curmir_incs)
			except (OSError, IOError), exc:
				Log.FatalError("Could not check if rdiff-backup is currently"
							   "running due to\n%s" % exc)
		assert len(curmir_incs) == 2, "Found too many current_mirror incs!"
		return 1

def checkdest_if_necessary(dest_rp):
	"""Check the destination dir if necessary.

	This can/should be run before an incremental backup.

	"""
	need_check = checkdest_need_check(dest_rp)
	if need_check == 1:
		Log("Previous backup seems to have failed, regressing "
			"destination now.", 2)
		dest_rp.conn.regress.Regress(dest_rp)



# Copyright 2002 Ben Escoto
#
# This file is part of rdiff-backup.
#
# rdiff-backup is free software; you can redistribute it and/or modify
# under the terms of the GNU General Public License as published by the
# Free Software Foundation; either version 2 of the License, or (at your
# option) any later version.
#
# rdiff-backup is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with rdiff-backup; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
# USA

"""Store and retrieve metadata in destination directory

The plan is to store metadata information for all files in the
destination directory in a special metadata file.  There are two
reasons for this:

1)  The filesystem of the mirror directory may not be able to handle
    types of metadata that the source filesystem can.  For instance,
    rdiff-backup may not have root access on the destination side, so
    cannot set uid/gid.  Or the source side may have ACLs and the
    destination side doesn't.

	Hopefully every file system can store binary data.  Storing
	metadata separately allows us to back up anything (ok, maybe
	strange filenames are still a problem).

2)  Metadata can be more quickly read from a file than it can by
    traversing the mirror directory over and over again.  In many
    cases most of rdiff-backup's time is spent compaing metadata (like
    file size and modtime), trying to find differences.  Reading this
    data sequentially from a file is significantly less taxing than
    listing directories and statting files all over the mirror
    directory.

The metadata is stored in a text file, which is a bunch of records
concatenated together.  Each record has the format:

File <filename>
  <field_name1> <value>
  <field_name2> <value>
  ...

Where the lines are separated by newlines.  See the code below for the
field names and values.

"""

from __future__ import generators
import re, gzip, os, binascii
import log, Globals, rpath, Time, robust, increment, static, rorpiter

class ParsingError(Exception):
	"""This is raised when bad or unparsable data is received"""
	pass

def carbonfile2string(cfile):
	"""Convert CarbonFile data to a string suitable for storing."""
	if not cfile: return "None"
	retvalparts = []
	retvalparts.append('creator:%s' % binascii.hexlify(cfile['creator']))
	retvalparts.append('type:%s' % binascii.hexlify(cfile['type']))
	retvalparts.append('location:%d,%d' % cfile['location'])
	retvalparts.append('flags:%d' % cfile['flags'])
	try: retvalparts.append('createDate:%d' % cfile['createDate'])
	except KeyError: log.Log("Writing pre-1.1.6 style metadata, without creation date", 9)
	return '|'.join(retvalparts)

def string2carbonfile(data):
	"""Re-constitute CarbonFile data from a string stored by 
	carbonfile2string."""
	retval = {}
	for component in data.split('|'):
		key, value = component.split(':')
		if key == 'creator':
			retval['creator'] = binascii.unhexlify(value)
		elif key == 'type':
			retval['type'] = binascii.unhexlify(value)
		elif key == 'location':
			a, b = value.split(',')
			retval['location'] = (int(a), int(b))
		elif key == 'flags':
			retval['flags'] = int(value)
		elif key == 'createDate':
			retval['createDate'] = int(value)
	return retval

def RORP2Record(rorpath):
	"""From RORPath, return text record of file's metadata"""
	str_list = ["File %s\n" % quote_path(rorpath.get_indexpath())]

	# Store file type, e.g. "dev", "reg", or "sym", and type-specific data
	type = rorpath.gettype()
	if type is None: type = "None"
	str_list.append("  Type %s\n" % type)
	if type == "reg":
		str_list.append("  Size %s\n" % rorpath.getsize())

		# If there is a resource fork, save it.
		if rorpath.has_resource_fork():
			if not rorpath.get_resource_fork(): rf = "None"
			else: rf = binascii.hexlify(rorpath.get_resource_fork())
			str_list.append("  ResourceFork %s\n" % (rf,))
                
		# If there is Carbon data, save it.
		if rorpath.has_carbonfile():
			cfile = carbonfile2string(rorpath.get_carbonfile())
			str_list.append("  CarbonFile %s\n" % (cfile,))

		# If file is hardlinked, add that information
		if Globals.preserve_hardlinks != 0:
			numlinks = rorpath.getnumlinks()
			if numlinks > 1:
				str_list.append("  NumHardLinks %s\n" % numlinks)
				str_list.append("  Inode %s\n" % rorpath.getinode())
				str_list.append("  DeviceLoc %s\n" % rorpath.getdevloc())

		# Save any hashes, if available
		if rorpath.has_sha1():
			str_list.append('  SHA1Digest %s\n' % rorpath.get_sha1())

	elif type == "None": return "".join(str_list)
	elif type == "dir" or type == "sock" or type == "fifo": pass
	elif type == "sym":
		str_list.append("  SymData %s\n" % quote_path(rorpath.readlink()))
	elif type == "dev":
		major, minor = rorpath.getdevnums()
		if rorpath.isblkdev(): devchar = "b"
		else:
			assert rorpath.ischardev()
			devchar = "c"
		str_list.append("  DeviceNum %s %s %s\n" % (devchar, major, minor))

	# Store time information
	if type != 'sym' and type != 'dev':
		str_list.append("  ModTime %s\n" % rorpath.getmtime())

	# Add user, group, and permission information
	uid, gid = rorpath.getuidgid()
	str_list.append("  Uid %s\n" % uid)
	str_list.append("  Uname %s\n" % (rorpath.getuname() or ":"))
	str_list.append("  Gid %s\n" % gid)
	str_list.append("  Gname %s\n" % (rorpath.getgname() or ":"))
	str_list.append("  Permissions %s\n" % rorpath.getperms())

	# Add long filename information
	if rorpath.has_alt_mirror_name():
		str_list.append("  AlternateMirrorName %s\n" %
						(rorpath.get_alt_mirror_name(),))
	elif rorpath.has_alt_inc_name():
		str_list.append("  AlternateIncrementName %s\n" %
						(rorpath.get_alt_inc_name(),))

	return "".join(str_list)

line_parsing_regexp = re.compile("^ *([A-Za-z0-9]+) (.+)$", re.M)
def Record2RORP(record_string):
	"""Given record_string, return RORPath

	For speed reasons, write the RORPath data dictionary directly
	instead of calling rorpath functions.  Profiling has shown this to
	be a time critical function.

	"""
	data_dict = {}
	for field, data in line_parsing_regexp.findall(record_string):
		if field == "File": index = quoted_filename_to_index(data)
		elif field == "Type":
			if data == "None": data_dict['type'] = None
			else: data_dict['type'] = data
		elif field == "Size": data_dict['size'] = long(data)
		elif field == "ResourceFork":
			if data == "None": data_dict['resourcefork'] = ""
			else: data_dict['resourcefork'] = binascii.unhexlify(data)
		elif field == "CarbonFile":
			if data == "None": data_dict['carbonfile'] = None
			else: data_dict['carbonfile'] = string2carbonfile(data)
		elif field == "SHA1Digest": data_dict['sha1'] = data
		elif field == "NumHardLinks": data_dict['nlink'] = int(data)
		elif field == "Inode": data_dict['inode'] = long(data)
		elif field == "DeviceLoc": data_dict['devloc'] = long(data)
		elif field == "SymData": data_dict['linkname'] = unquote_path(data)
		elif field == "DeviceNum":
			devchar, major_str, minor_str = data.split(" ")
			data_dict['devnums'] = (devchar, int(major_str), int(minor_str))
		elif field == "ModTime": data_dict['mtime'] = long(data)
		elif field == "Uid": data_dict['uid'] = int(data)
		elif field == "Gid": data_dict['gid'] = int(data)
		elif field == "Uname":
			if data == ":" or data == 'None': data_dict['uname'] = None
			else: data_dict['uname'] = data
		elif field == "Gname":
			if data == ':' or data == 'None': data_dict['gname'] = None
			else: data_dict['gname'] = data
		elif field == "Permissions": data_dict['perms'] = int(data)
		elif field == "AlternateMirrorName": data_dict['mirrorname'] = data
		elif field == "AlternateIncrementName": data_dict['incname'] = data
		else: log.Log("Unknown field in line '%s %s'" % (field, data), 2)
	return rpath.RORPath(index, data_dict)

chars_to_quote = re.compile("\\n|\\\\")
def quote_path(path_string):
	"""Return quoted verson of path_string

	Because newlines are used to separate fields in a record, they are
	replaced with \n.  Backslashes become \\ and everything else is
	left the way it is.

	"""
	def replacement_func(match_obj):
		"""This is called on the match obj of any char that needs quoting"""
		char = match_obj.group(0)
		if char == "\n": return "\\n"
		elif char == "\\": return "\\\\"
		assert 0, "Bad char %s needs quoting" % char
	return chars_to_quote.sub(replacement_func, path_string)

def unquote_path(quoted_string):
	"""Reverse what was done by quote_path"""
	def replacement_func(match_obj):
		"""Unquote match obj of two character sequence"""
		two_chars = match_obj.group(0)
		if two_chars == "\\n": return "\n"
		elif two_chars == "\\\\": return "\\"
		log.Log("Warning, unknown quoted sequence %s found" % two_chars, 2)
		return two_chars
	return re.sub("\\\\n|\\\\\\\\", replacement_func, quoted_string)

def quoted_filename_to_index(quoted_filename):
	"""Return tuple index given quoted filename"""
	if quoted_filename == '.': return ()
	else: return tuple(unquote_path(quoted_filename).split('/'))

class FlatExtractor:
	"""Controls iterating objects from flat file"""

	# Set this in subclass.  record_boundary_regexp should match
	# beginning of next record.  The first group should start at the
	# beginning of the record.  The second group should contain the
	# (possibly quoted) filename.
	record_boundary_regexp = None

	# Set in subclass to function that converts text record to object
	record_to_object = None

	def __init__(self, fileobj):
		self.fileobj = fileobj # holds file object we are reading from
		self.buf = "" # holds the next part of the file
		self.at_end = 0 # True if we are at the end of the file
		self.blocksize = 32 * 1024

	def get_next_pos(self):
		"""Return position of next record in buffer, or end pos if none"""
		while 1:
			m = self.record_boundary_regexp.search(self.buf, 1)
			if m: return m.start(1)
			else: # add next block to the buffer, loop again
				newbuf = self.fileobj.read(self.blocksize)
				if not newbuf:
					self.at_end = 1
					return len(self.buf)
				else: self.buf += newbuf

	def iterate(self):
		"""Return iterator that yields all objects with records"""
		for record in self.iterate_records():
			try: yield self.record_to_object(record)
			except (ParsingError, ValueError), e:
				if self.at_end: break # Ignore whitespace/bad records at end
				log.Log("Error parsing flat file: %s" % (e,), 2)

	def iterate_records(self):
		"""Yield all text records in order"""
		while 1:
			next_pos = self.get_next_pos()
			if self.at_end:
				if next_pos: yield self.buf[:next_pos]
				break
			yield self.buf[:next_pos]
			self.buf = self.buf[next_pos:]
		assert not self.fileobj.close()

	def skip_to_index(self, index):
		"""Scan through the file, set buffer to beginning of index record

		Here we make sure that the buffer always ends in a newline, so
		we will not be splitting lines in half.

		"""
		assert not self.buf or self.buf.endswith("\n")
		while 1:
			self.buf = self.fileobj.read(self.blocksize)
			self.buf += self.fileobj.readline()
			if not self.buf:
				self.at_end = 1
				return
			while 1:
				m = self.record_boundary_regexp.search(self.buf)
				if not m: break
				cur_index = self.filename_to_index(m.group(2))
				if cur_index >= index:
					self.buf = self.buf[m.start(1):]
					return
				else: self.buf = self.buf[m.end(1):]

	def iterate_starting_with(self, index):
		"""Iterate objects whose index starts with given index"""
		self.skip_to_index(index)
		if self.at_end: return
		while 1:
			next_pos = self.get_next_pos()
			try: obj = self.record_to_object(self.buf[:next_pos])
			except (ParsingError, ValueError), e:
				log.Log("Error parsing metadata file: %s" % (e,), 2)
			else:
				if obj.index[:len(index)] != index: break
				yield obj
			if self.at_end: break
			self.buf = self.buf[next_pos:]
		assert not self.fileobj.close()

	def filename_to_index(self, filename):
		"""Translate filename, possibly quoted, into an index tuple

		The filename is the first group matched by
		regexp_boundary_regexp.

		"""
		assert 0 # subclass


class RorpExtractor(FlatExtractor):
	"""Iterate rorps from metadata file"""
	record_boundary_regexp = re.compile("(?:\\n|^)(File (.*?))\\n")
	record_to_object = staticmethod(Record2RORP)
	filename_to_index = staticmethod(quoted_filename_to_index)


class FlatFile:
	"""Manage a flat (probably text) file containing info on various files

	This is used for metadata information, and possibly EAs and ACLs.
	The main read interface is as an iterator.  The storage format is
	a flat, probably compressed file, so random access is not
	recommended.

	"""
	rp, fileobj, mode = None, None, None
	_buffering_on = 1 # Buffering may be useful because gzip writes are slow
	_record_buffer, _max_buffer_size = None, 100
	_extractor = FlatExtractor # Override to class that iterates objects
	_object_to_record = None # Set to function converting object to record
	_prefix = None # Set to required prefix
	def __init__(self, rp_base, mode, check_path = 1, compress = 1,
				 callback = None):
		"""Open rp (or rp+'.gz') for reading ('r') or writing ('w')

		If callback is available, it will be called on the rp upon
		closing (because the rp may not be known in advance).

		"""
		self.mode = mode
		self.callback = callback
		self._record_buffer = []
		if check_path:
			assert (rp_base.isincfile() and
					rp_base.getincbase_str() == self._prefix), rp_base
			compress = 1
		if mode == 'r':
			self.rp = rp_base
			self.fileobj = self.rp.open("rb", compress)
		else:
			assert mode == 'w'
			if compress and check_path and not rp_base.isinccompressed():
				def callback(rp): self.rp = rp
				self.fileobj = rpath.MaybeGzip(rp_base, callback)
			else:
				self.rp = rp_base
				assert not self.rp.lstat(), self.rp
				self.fileobj = self.rp.open("wb", compress = compress)

	def write_record(self, record):
		"""Write a (text) record into the file"""
		if self._buffering_on:
			self._record_buffer.append(record)
			if len(self._record_buffer) >= self._max_buffer_size:
				self.fileobj.write("".join(self._record_buffer))
				self._record_buffer = []
		else: self.fileobj.write(record)

	def write_object(self, object):
		"""Convert one object to record and write to file"""
		self.write_record(self._object_to_record(object))

	def get_objects(self, restrict_index = None):
		"""Return iterator of objects records from file rp"""
		if not restrict_index: return self._extractor(self.fileobj).iterate()
		extractor = self._extractor(self.fileobj)
		return extractor.iterate_starting_with(restrict_index)

	def get_records(self):
		"""Return iterator of text records"""
		return self._extractor(self.fileobj).iterate_records()

	def close(self):
		"""Close file, for when any writing is done"""
		assert self.fileobj, "File already closed"
		if self._buffering_on and self._record_buffer: 
			self.fileobj.write("".join(self._record_buffer))
			self._record_buffer = []
		result = self.fileobj.close()
		self.fileobj = None
		self.rp.fsync_with_dir()
		self.rp.setdata()
		if self.callback: self.callback(self.rp)
		return result

class MetadataFile(FlatFile):
	"""Store/retrieve metadata from mirror_metadata as rorps"""
	_prefix = "mirror_metadata"
	_extractor = RorpExtractor
	_object_to_record = staticmethod(RORP2Record)


class CombinedWriter:
	"""Used for simultaneously writting metadata, eas, and acls"""
	def __init__(self, metawriter, eawriter, aclwriter, winaclwriter):
		self.metawriter = metawriter
		self.eawriter, self.aclwriter, self.winaclwriter = \
				eawriter, aclwriter, winaclwriter # these can be None

	def write_object(self, rorp):
		"""Write information in rorp to all the writers"""
		self.metawriter.write_object(rorp)
		if self.eawriter and not rorp.get_ea().empty():
			self.eawriter.write_object(rorp.get_ea())
		if self.aclwriter and not rorp.get_acl().is_basic():
			self.aclwriter.write_object(rorp.get_acl())
		if self.winaclwriter:
			self.winaclwriter.write_object(rorp.get_win_acl())

	def close(self):
		self.metawriter.close()
		if self.eawriter: self.eawriter.close()
		if self.aclwriter: self.aclwriter.close()
		if self.winaclwriter: self.winaclwriter.close()


class Manager:
	"""Read/Combine/Write metadata files by time"""
	meta_prefix = 'mirror_metadata'
	acl_prefix = 'access_control_lists'
	ea_prefix = 'extended_attributes'
	wacl_prefix = 'win_access_control_lists'

	def __init__(self):
		"""Set listing of rdiff-backup-data dir"""
		self.rplist = []
		self.timerpmap, self.prefixmap = {}, {}
		for filename in Globals.rbdir.listdir():
			rp = Globals.rbdir.append(filename)
			if rp.isincfile(): self.add_incrp(rp)
				
	def add_incrp(self, rp):
		"""Add rp to list of inc rps in the rbdir"""
		assert rp.isincfile(), rp
		self.rplist.append(rp)
		time = rp.getinctime()
		if self.timerpmap.has_key(time):
			self.timerpmap[time].append(rp)
		else: self.timerpmap[time] = [rp]

		incbase = rp.getincbase_str()
		if self.prefixmap.has_key(incbase): self.prefixmap[incbase].append(rp)
		else: self.prefixmap[incbase] = [rp]

	def _iter_helper(self, prefix, flatfileclass, time, restrict_index):
		"""Used below to find the right kind of file by time"""
		if not self.timerpmap.has_key(time): return None
		for rp in self.timerpmap[time]:
			if rp.getincbase_str() == prefix:
				return flatfileclass(rp, 'r').get_objects(restrict_index)
		return None

	def get_meta_at_time(self, time, restrict_index):
		"""Return iter of metadata rorps at given time (or None)"""
		return self._iter_helper(self.meta_prefix, MetadataFile,
								 time, restrict_index)

	def get_eas_at_time(self, time, restrict_index):
		"""Return Extended Attributes iter at given time (or None)"""
		return self._iter_helper(self.ea_prefix,
					  eas_acls.ExtendedAttributesFile, time, restrict_index)

	def get_acls_at_time(self, time, restrict_index):
		"""Return ACLs iter at given time from recordfile (or None)"""
		return self._iter_helper(self.acl_prefix,
					  eas_acls.AccessControlListFile, time, restrict_index)

	def get_win_acls_at_time(self, time, restrict_index):
		"""Return WACLs iter at given time from recordfile (or None)"""
		return self._iter_helper(self.wacl_prefix,
					  win_acls.WinAccessControlListFile, time, restrict_index)

	def GetAtTime(self, time, restrict_index = None):
		"""Return combined metadata iter with ea/acl info if necessary"""
		cur_iter = self.get_meta_at_time(time, restrict_index)
		if not cur_iter:
			log.Log("Warning, could not find mirror_metadata file.\n"
					"Metadata will be read from filesystem instead.", 2)
			return None

		if Globals.acls_active:
			acl_iter = self.get_acls_at_time(time, restrict_index)
			if not acl_iter:
				log.Log("Warning: Access Control List file not found", 2)
				acl_iter = iter([])
			cur_iter = eas_acls.join_acl_iter(cur_iter, acl_iter)
		if Globals.eas_active:
			ea_iter = self.get_eas_at_time(time, restrict_index)
			if not ea_iter:
				log.Log("Warning: Extended Attributes file not found", 2)
				ea_iter = iter([])
			cur_iter = eas_acls.join_ea_iter(cur_iter, ea_iter)
		if Globals.win_acls_active:
			wacl_iter = self.get_win_acls_at_time(time, restrict_index)
			if not wacl_iter:
				log.Log("Warning: Windows Access Control List file not"
						" found.", 2)
				wacl_iter = iter([])
			cur_iter = win_acls.join_wacl_iter(cur_iter, wacl_iter)

		return cur_iter

	def _writer_helper(self, prefix, flatfileclass, typestr, time):
		"""Used in the get_xx_writer functions, returns a writer class"""
		if time is None: timestr = Time.curtimestr
		else: timestr = Time.timetostring(time)		
		filename = '%s.%s.%s' % (prefix, timestr, typestr)
		rp = Globals.rbdir.append(filename)
		assert not rp.lstat(), "File %s already exists!" % (rp.path,)
		assert rp.isincfile()
		return flatfileclass(rp, 'w', callback = self.add_incrp)

	def get_meta_writer(self, typestr, time):
		"""Return MetadataFile object opened for writing at given time"""
		return self._writer_helper(self.meta_prefix, MetadataFile,
								   typestr, time)

	def get_ea_writer(self, typestr, time):
		"""Return ExtendedAttributesFile opened for writing"""
		return self._writer_helper(self.ea_prefix,
						 eas_acls.ExtendedAttributesFile, typestr, time)

	def get_acl_writer(self, typestr, time):
		"""Return AccessControlListFile opened for writing"""
		return self._writer_helper(self.acl_prefix,
						 eas_acls.AccessControlListFile, typestr, time)

	def get_win_acl_writer(self, typestr, time):
		"""Return WinAccessControlListFile opened for writing"""
		return self._writer_helper(self.wacl_prefix,
						 win_acls.WinAccessControlListFile, typestr, time)

	def GetWriter(self, typestr = 'snapshot', time = None):
		"""Get a writer object that can write meta and possibly acls/eas"""
		metawriter = self.get_meta_writer(typestr, time)
		if not Globals.eas_active and not Globals.acls_active and \
				not Globals.win_acls_active:
			return metawriter # no need for a CombinedWriter

		if Globals.eas_active: ea_writer = self.get_ea_writer(typestr, time)
		else: ea_writer = None
		if Globals.acls_active: acl_writer = self.get_acl_writer(typestr, time)
		else: acl_writer = None
		if Globals.win_acls_active: win_acl_writer = \
				self.get_win_acl_writer(typestr, time)
		else: win_acl_writer = None
		return CombinedWriter(metawriter, ea_writer, acl_writer, win_acl_writer)

class PatchDiffMan(Manager):
	"""Contains functions for patching and diffing metadata

	To save space, we can record a full list of only the most recent
	metadata, using the normal rdiff-backup reverse increment
	strategy.  Instead of using librsync to compute diffs, though, we
	use our own technique so that the diff files are still
	hand-editable.

	A mirror_metadata diff has the same format as a mirror_metadata
	snapshot.  If the record for an index is missing from the diff, it
	indicates no change from the original.  If it is present it
	replaces the mirror_metadata entry, unless it has Type None, which
	indicates the record should be deleted from the original.

	"""
	max_diff_chain = 9 # After this many diffs, make a new snapshot

	def get_diffiter(self, new_iter, old_iter):
		"""Iterate meta diffs of new_iter -> old_iter"""
		for new_rorp, old_rorp in rorpiter.Collate2Iters(new_iter, old_iter):
			if not old_rorp: yield rpath.RORPath(new_rorp.index)
			elif not new_rorp or new_rorp.data != old_rorp.data:
				# exact compare here, can't use == on rorps
				yield old_rorp

	def sorted_prefix_inclist(self, prefix, min_time = 0):
		"""Return reverse sorted (by time) list of incs with given prefix"""
		if not self.prefixmap.has_key(prefix): return []
		sortlist = [(rp.getinctime(), rp) for rp in self.prefixmap[prefix]]
		sortlist.sort()
		sortlist.reverse()
		return [rp for (time, rp) in sortlist if time >= min_time]

	def check_needs_diff(self):
		"""Check if we should diff, returns (new, old) rps, or (None, None)"""
		inclist = self.sorted_prefix_inclist('mirror_metadata')
		assert len(inclist) >= 1
		if len(inclist) == 1: return (None, None)
		newrp, oldrp = inclist[:2]
		assert newrp.getinctype() == oldrp.getinctype() == 'snapshot'

		chainlen = 1
		for rp in inclist[2:]:
			if rp.getinctype() != 'diff': break
			chainlen += 1
		if chainlen >= self.max_diff_chain: return (None, None)
		return (newrp, oldrp)

	def ConvertMetaToDiff(self):
		"""Replace a mirror snapshot with a diff if it's appropriate"""
		newrp, oldrp = self.check_needs_diff()
		if not newrp: return
		log.Log("Writing mirror_metadata diff", 6)

		diff_writer = self.get_meta_writer('diff', oldrp.getinctime())
		new_iter = MetadataFile(newrp, 'r').get_objects()
		old_iter = MetadataFile(oldrp, 'r').get_objects()
		for diff_rorp in self.get_diffiter(new_iter, old_iter):
			diff_writer.write_object(diff_rorp)
		diff_writer.close() # includes sync
		oldrp.delete()

	def get_meta_at_time(self, time, restrict_index):
		"""Get metadata rorp iter, possibly by patching with diffs"""
		meta_iters = [MetadataFile(rp, 'r').get_objects(restrict_index)
					  for rp in self.relevant_meta_incs(time)]
		if not meta_iters: return None
		if len(meta_iters) == 1: return meta_iters[0]
		return self.iterate_patched_meta(meta_iters)

	def relevant_meta_incs(self, time):
		"""Return list [snapshotrp, diffrps ...] time sorted"""
		inclist = self.sorted_prefix_inclist('mirror_metadata', min_time=time)
		if not inclist: return inclist
		assert inclist[-1].getinctime() == time, inclist[-1]
		for i in range(len(inclist)-1, -1, -1):
			if inclist[i].getinctype() == 'snapshot':
				return inclist[i:]
		assert 0, "Inclist %s contains no snapshots" % (inclist,)

	def iterate_patched_meta(self, meta_iter_list):
		"""Return an iter of metadata rorps by combining the given iters

		The iters should be given as a list/tuple in reverse
		chronological order.  The earliest rorp in each iter will
		supercede all the later ones.

		"""
		for meta_tuple in rorpiter.CollateIterators(*meta_iter_list):
			for i in range(len(meta_tuple)-1, -1, -1):
				if meta_tuple[i]:
					if meta_tuple[i].lstat(): yield meta_tuple[i]
					break # move to next index
			else: assert 0, "No valid rorps"

ManagerObj = None # Set this later to Manager instance
def SetManager():
	global ManagerObj
	ManagerObj = PatchDiffMan()
	return ManagerObj


import eas_acls, win_acls # put at bottom to avoid python circularity bug



# Copyright 2002 Ben Escoto
#
# This file is part of rdiff-backup.
#
# rdiff-backup is free software; you can redistribute it and/or modify
# under the terms of the GNU General Public License as published by the
# Free Software Foundation; either version 2 of the License, or (at your
# option) any later version.
#
# rdiff-backup is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with rdiff-backup; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
# USA

"""Catch various exceptions given system call"""

import errno, signal, exceptions
import librsync, C, static, rpath, Globals, log, statistics, connection

def check_common_error(error_handler, function, args = []):
	"""Apply function to args, if error, run error_handler on exception

	This uses the catch_error predicate below to only catch
	certain exceptions which seems innocent enough.

	"""
	try: return function(*args)
	except (Exception, KeyboardInterrupt, SystemExit), exc:
		TracebackArchive.add([function] + list(args))
		if catch_error(exc):
			log.Log.exception()
			conn = Globals.backup_writer
			if conn is not None: conn.statistics.record_error()
			if error_handler: return error_handler(exc, *args)
			else: return None
		if is_routine_fatal(exc): log.Log.exception(1, 6)
		else: log.Log.exception(1, 2)
		raise

def catch_error(exc):
	"""Return true if exception exc should be caught"""
	for exception_class in (rpath.SkipFileException, rpath.RPathException,
							librsync.librsyncError, C.UnknownFileTypeError):
		if isinstance(exc, exception_class): return 1
	if (isinstance(exc, EnvironmentError) and
		# the invalid mode shows up in backups of /proc for some reason
		(exc[0] in ('invalid mode: rb', 'Not a gzipped file') or
		 errno.errorcode.has_key(exc[0]) and
		 errno.errorcode[exc[0]] in ('EPERM', 'ENOENT', 'EACCES', 'EBUSY',
									 'EEXIST', 'ENOTDIR', 'EILSEQ',
									 'ENAMETOOLONG', 'EINTR', 'ESTALE',
									 'ENOTEMPTY', 'EIO', 'ETXTBSY',
									 'ESRCH', 'EINVAL', 'EDEADLOCK',
									 'EDEADLK', 'EOPNOTSUPP', 'ETIMEDOUT'))):
		return 1
	return 0

def is_routine_fatal(exc):
	"""Return string if exception is non-error unrecoverable, None otherwise

	Used to suppress a stack trace for exceptions like keyboard
	interrupts or connection drops.  Return value is string to use as
	an exit message.

	"""
	if isinstance(exc, exceptions.KeyboardInterrupt):
		return "User abort"
	elif isinstance(exc, connection.ConnectionError):
		return "Lost connection to the remote system"
	elif isinstance(exc, SignalException):
		return "Killed with signal %s" % (exc,)
	elif isinstance(exc, EnvironmentError) and exc.errno == errno.ENOTCONN:
		return ("Filesystem reports connection failure:\n%s" % exc)
	return None

def get_error_handler(error_type):
	"""Return error handler function that can be used above

	Function will just log error to the error_log and then return
	None.  First two arguments must be the exception and then an rp
	(from which the filename will be extracted).

	"""
	def error_handler(exc, rp, *args):
		log.ErrorLog.write_if_open(error_type, rp, exc)
		return 0
	return error_handler

def listrp(rp):
	"""Like rp.listdir() but return [] if error, and sort results"""
	def error_handler(exc):
		log.Log("Error listing directory %s" % rp.path, 2)
		return []
	dir_listing = check_common_error(error_handler, rp.listdir)
	dir_listing.sort()
	return dir_listing

def signal_handler(signum, frame):
	"""This is called when signal signum is caught"""
	raise SignalException(signum)

def install_signal_handlers():
	"""Install signal handlers on current connection"""
	signals = [signal.SIGTERM, signal.SIGINT]
	try:
		signals.extend([signal.SIGHUP, signal.SIGQUIT])
	except AttributeError:
	    pass
	for signum in signals:
		signal.signal(signum, signal_handler)


class SignalException(Exception):
	"""SignalException(signum) means signal signum has been received"""
	pass


class TracebackArchive:
	"""Save last 10 caught exceptions, so they can be printed if fatal"""
	_traceback_strings = []
	def add(cls, extra_args = []):
		"""Add most recent exception to archived list

		If extra_args are present, convert to strings and add them as
		extra information to same traceback archive.

		"""
		cls._traceback_strings.append(log.Log.exception_to_string(extra_args))
		if len(cls._traceback_strings) > 10:
			cls._traceback_strings = cls._traceback_strings[:10]

	def log(cls):
		"""Print all exception information to log file"""
		if cls._traceback_strings:
			log.Log("------------ Old traceback info -----------\n%s\n"
					"-------------------------------------------" %
					("\n".join(cls._traceback_strings),), 3)

static.MakeClass(TracebackArchive)




# Copyright 2002 2005 Ben Escoto
#
# This file is part of rdiff-backup.
#
# rdiff-backup is free software; you can redistribute it and/or modify
# under the terms of the GNU General Public License as published by the
# Free Software Foundation; either version 2 of the License, or (at your
# option) any later version.
#
# rdiff-backup is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with rdiff-backup; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
# USA

"""Preserve and restore hard links

If the preserve_hardlinks option is selected, linked files in the
source directory will be linked in the mirror directory.  Linked files
are treated like any other with respect to incrementing, but their
link status can be retrieved because their device location and inode #
is written in the metadata file.

All these functions are meant to be executed on the mirror side.  The
source side should only transmit inode information.

"""

from __future__ import generators
import Globals, Time, log, robust, errno

# The keys in this dictionary are (inode, devloc) pairs.  The values
# are a pair (index, remaining_links, dest_key, sha1sum) where index
# is the rorp index of the first such linked file, remaining_links is
# the number of files hard linked to this one we may see, and key is
# either (dest_inode, dest_devloc) or None, and represents the
# hardlink info of the existing file on the destination.  Finally
# sha1sum is the hash of the file if it exists, or None.
_inode_index = None

def initialize_dictionaries():
	"""Set all the hard link dictionaries to empty"""
	global _inode_index
	_inode_index = {}

def clear_dictionaries():
	"""Delete all dictionaries"""
	global _inode_index
	_inode_index = None

def get_inode_key(rorp):
	"""Return rorp's key for _inode_ dictionaries"""
	return (rorp.getinode(), rorp.getdevloc())

def add_rorp(rorp, dest_rorp = None):
	"""Process new rorp and update hard link dictionaries"""
	if not rorp.isreg() or rorp.getnumlinks() < 2: return None
	rp_inode_key = get_inode_key(rorp)
	if not _inode_index.has_key(rp_inode_key):
		if not dest_rorp: dest_key = None
		elif dest_rorp.getnumlinks() == 1: dest_key = "NA"
		else: dest_key = get_inode_key(dest_rorp)
		digest = rorp.has_sha1() and rorp.get_sha1() or None
		_inode_index[rp_inode_key] = (rorp.index, rorp.getnumlinks(),
									  dest_key, digest)
	return rp_inode_key

def del_rorp(rorp):
	"""Remove rorp information from dictionary if seen all links"""
	if not rorp.isreg() or rorp.getnumlinks() < 2: return
	rp_inode_key = get_inode_key(rorp)
	val = _inode_index.get(rp_inode_key)
	if not val: return
	index, remaining, dest_key, digest = val
	if remaining == 1:
		del _inode_index[rp_inode_key]
		return 1
	else:
		_inode_index[rp_inode_key] = (index, remaining-1, dest_key, digest)
		return 0

def rorp_eq(src_rorp, dest_rorp):
	"""Compare hardlinked for equality

	Return false if dest_rorp is linked differently, which can happen
	if dest is linked more than source, or if it is represented by a
	different inode.

	"""
	if (not src_rorp.isreg() or not dest_rorp.isreg() or
		src_rorp.getnumlinks() == dest_rorp.getnumlinks() == 1):
		return 1 # Hard links don't apply

	if src_rorp.getnumlinks() < dest_rorp.getnumlinks(): return 0
	src_key = get_inode_key(src_rorp)
	index, remaining, dest_key, digest = _inode_index[src_key]
	if dest_key == "NA":
		# Allow this to be ok for first comparison, but not any
		# subsequent ones
		_inode_index[src_key] = (index, remaining, None, None)
		return 1
	try:
		return dest_key == get_inode_key(dest_rorp)
	except KeyError:
		return 0 # Inode key might be missing if the metadata file is corrupt

def islinked(rorp):
	"""True if rorp's index is already linked to something on src side"""
	if not rorp.getnumlinks() > 1: return 0
	dict_val = _inode_index.get(get_inode_key(rorp))
	if not dict_val: return 0
	return dict_val[0] != rorp.index # If equal, then rorp is first

def get_link_index(rorp):
	"""Return first index on target side rorp is already linked to"""
	return _inode_index[get_inode_key(rorp)][0]

def get_sha1(rorp):
	"""Return sha1 digest of what rorp is linked to"""
	return _inode_index[get_inode_key(rorp)][3]

def link_rp(diff_rorp, dest_rpath, dest_root = None):
	"""Make dest_rpath into a link using link flag in diff_rorp"""
	if not dest_root: dest_root = dest_rpath # use base of dest_rpath
	dest_link_rpath = dest_root.new_index(diff_rorp.get_link_flag())
	try: dest_rpath.hardlink(dest_link_rpath.path)
	except EnvironmentError, exc:
		# This can happen if the source of dest_link_rpath was deleted
		# after it's linking info was recorded but before
		# dest_link_rpath was written.
		if errno.errorcode[exc[0]] == 'ENOENT':
			dest_rpath.touch() # This will cause an UpdateError later
		else: raise Exception("EnvironmentError '%s' linking %s to %s" %
							  (exc, dest_rpath.path, dest_link_rpath.path))

